const ec$2 = new TextEncoder();
const ALPHABETS$1 = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    }
];
function getAlphabet$1(name) {
    for (const alpha of ALPHABETS$1) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1$1(data, charset, padding = false) {
    if (typeof data === 'string')
        data = ec$2.encode(data);
    const alphabet = getAlphabet$1(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1$1(encoded, charset) {
    const alphabet = getAlphabet$1(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        if (!(c > 0 || (b.length ^ i) > 0))
            b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return new Uint8Array(b);
}
const BaseX$1 = {
    encode: encode$1$1,
    decode: decode$1$1
};

const CHARSET$1 = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR$1 = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const encodings$1 = {
    BECH32: 'bech32',
    BECH32M: 'bech32m'
};
function getEncodingConst$1(enc) {
    switch (enc) {
        case encodings$1.BECH32:
            return 1;
        case encodings$1.BECH32M:
            return 0x2bc830a3;
        default:
            throw new Error(`Unrecognized encoding: ${enc}`);
    }
}
function polymod$1(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) > 0) {
                chk ^= GENERATOR$1[i];
            }
        }
    }
    return chk;
}
function hrpExpand$1(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function convertBits$1(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            return [];
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        return [];
    }
    return ret;
}
function verifyChecksum$1(hrp, data, enc) {
    const combined = hrpExpand$1(hrp).concat(data);
    return polymod$1(combined) === getEncodingConst$1(enc);
}
function createChecksum$1(hrp, data, enc) {
    const values = hrpExpand$1(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod$1(values) ^ getEncodingConst$1(enc);
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function b32encode$1(hrp, data, enc) {
    const combined = data.concat(createChecksum$1(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET$1.charAt(combined[p]);
    }
    return ret;
}
function b32decode$1(bechstr, version) {
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    if (!checkBounds$1(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos$1(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET$1.indexOf(bechstr.charAt(p));
        if (d === -1) {
            return { hrp: null, data: [0xFF] };
        }
        data.push(d);
    }
    return (verifyChecksum$1(hrp, data, enc))
        ? { hrp, data: data.slice(0, data.length - 6) }
        : { hrp: null, data: [0xFF] };
}
function checkBounds$1(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    return !(hasLower && hasUpper);
}
function checkSeparatorPos$1(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function encode$6(data, hrp = 'bch', version = 0) {
    const dat = [version, ...convertBits$1([...data], 8, 5)];
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    const str = b32encode$1(hrp, dat, enc);
    decode$6(str, version);
    return str;
}
function decode$6(str, version = 0) {
    const hrp = str.split('1', 1)[0];
    const { hrp: hrpgot, data } = b32decode$1(str, version);
    const decoded = convertBits$1(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        case (data[0] === 0 && length !== 20 && length !== 32):
            throw new Error('Decoded string does not match version 0 spec.');
        case (data[0] === 0 && version !== 0):
            throw new Error('Decoded version bit does not match.');
        case (data[0] !== 0 && version !== 1):
            throw new Error('Decoded version bit does not match.');
        default:
            return Uint8Array.from(decoded);
    }
}
const Bech32$1 = {
    encode: encode$6,
    decode: decode$6
};

const BASE64_ALPHABET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
const B64URL_ALPHABET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_';
const ec$1$1 = new TextEncoder();
function b64encode(input, url = false) {
    const buffer = [];
    const encoding = (url) ? B64URL_ALPHABET : BASE64_ALPHABET;
    if (typeof input === 'string')
        input = ec$1$1.encode(input);
    const n = 1;
    let out = '';
    let c = 0;
    let j = 1;
    let l = 0;
    let s = 0;
    for (let i = 0; i < input.length; i++) {
        c = input[i];
        for (let k = n - 1; k >= 0; k--) {
            buffer[k] = c & 0xff;
            c >>= 8;
        }
        for (let m = 0; m < n; m++) {
            l = ((l << 8) & 0xff00) | buffer[m];
            s = (0x3f << (2 * j)) & l;
            l -= s;
            out += encoding.charAt(s >> (2 * j));
            j++;
            if (j === 4) {
                out += encoding.charAt(l & 0x3f);
                j = 1;
            }
        }
    }
    switch (j) {
        case 2:
            s = 0x3f & (16 * l);
            out += encoding.charAt(s);
            out += '==';
            break;
        case 3:
            s = 0x3f & (4 * l);
            out += encoding.charAt(s);
            out += '=';
            break;
    }
    return out;
}
function b64decode(input, url = false) {
    const out = [];
    const map = {};
    const encoding = (url) ? B64URL_ALPHABET : BASE64_ALPHABET;
    for (let p = 0; p < encoding.length; p++) {
        map[encoding.charAt(p)] = p;
    }
    let l = 0;
    let i = 0;
    let j = 0;
    let c = 0;
    let end = input.length;
    for (let p = 0; p < 2; p++) {
        if (input.charAt(end - 1) === '=') {
            end--;
        }
        else {
            break;
        }
    }
    for (i = 0; i < end; i++) {
        l <<= 6;
        c = map[input.charAt(i)];
        l |= (c & 0x3f);
        if (j === 0) {
            j++;
            continue;
        }
        out.push(l >> (2 * (3 - j)));
        l &= ~(0xff << (2 * (3 - j)));
        j = ++j % 4;
    }
    return Uint8Array.of(...out);
}
const Base64 = {
    encode: b64encode,
    decode: b64decode
};
const B64URL = {
    encode: (data) => b64encode(data, true),
    decode: (data) => b64decode(data, true)
};

const ec$3 = new TextEncoder();
const dc$1 = new TextDecoder();
function strToBytes$1(str) {
    return ec$3.encode(str);
}
function hexToBytes$2(str) {
    const bytes = new Uint8Array(str.length / 2);
    let i, idx = 0;
    if (str.match(/[^a-fA-f0-9]/) !== null) {
        throw new TypeError('Invalid hex string: ' + str);
    }
    if (str.length % 2 > 0) {
        throw new Error(`Hex string length is uneven: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return bytes;
}
function numToBytes$1(num) {
    if (num === 0)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return new Uint8Array(bytes);
}
function binaryToBytes(binary) {
    const bytes = [];
    if (binary.length % 8 !== 0) {
        throw new Error('Binary array is invalid length: ' + String(binary.length));
    }
    for (let i = 0; i < binary.length; i += 8) {
        const byte = binary.slice(i, i + 8).map(e => String(e)).join('');
        bytes.push(parseInt(byte, 2));
    }
    return new Uint8Array(bytes);
}
function bytesToBinary(bytes) {
    const binary = [];
    for (const num of bytes) {
        const bits = num
            .toString(2)
            .padStart(8, '0')
            .split('')
            .map(e => parseInt(e));
        binary.push(...bits);
    }
    return binary;
}
function bigToBytes$1(big) {
    if (big === 0n)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return new Uint8Array(bytes);
}
function bytesToStr$1(bytes) {
    return dc$1.decode(bytes);
}
function bytesToHex$2(bytes) {
    const chars = new Array(bytes.length);
    for (let i = 0; i < bytes.length; i++) {
        chars.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return chars.join('');
}
function bytesToNum$1(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig$1(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer$1(value, bytes = true) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    switch (typeof value) {
        case 'bigint':
            return bigToBytes$1(value);
        case 'boolean':
            return Uint8Array.of(value ? 1 : 0);
        case 'number':
            return numToBytes$1(value);
        case 'string':
            return (bytes)
                ? hexToBytes$2(value)
                : ec$3.encode(value);
        default:
            throw TypeError('Unsupported format:' + String(typeof value));
    }
}

function bytesToBigInt$1(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes$1(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML$1 = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR$1 = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL$1 = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR$1 = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL$1 = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR$1 = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi$1(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol$1(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress$1(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt$1(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML$1[i])];
        kl = KL$1[Number(rnd)];
        mr = x[Number(MR$1[i])];
        kr = KR$1[Number(rnd)];
        al = rol$1(al + fi$1(bl, cl, dl, rnd) + ml + kl, RL$1[i]) + el;
        elt = el;
        el = dl;
        dl = rol$1(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol$1(ar + fi$1(br, cr, dr, 4n - rnd) + mr + kr, RR$1[i]) + er;
        ert = er;
        er = dr;
        dr = rol$1(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160$1(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress$1(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes$1(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress$1(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes$1(num, 4));
    }
    return Uint8Array.from(ret);
}

const digestLength$1 = 32;
const blockSize$1 = 64;
const K$1 = new Uint32Array([
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b,
    0x59f111f1, 0x923f82a4, 0xab1c5ed5, 0xd807aa98, 0x12835b01,
    0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7,
    0xc19bf174, 0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
    0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da, 0x983e5152,
    0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147,
    0x06ca6351, 0x14292967, 0x27b70a85, 0x2e1b2138, 0x4d2c6dfc,
    0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819,
    0xd6990624, 0xf40e3585, 0x106aa070, 0x19a4c116, 0x1e376c08,
    0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f,
    0x682e6ff3, 0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
    0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
]);
function hashBlocks$1(w, v, p, pos, len) {
    let a, b, c, d, e, f, g, h, u, i, j, t1, t2;
    while (len >= 64) {
        a = v[0];
        b = v[1];
        c = v[2];
        d = v[3];
        e = v[4];
        f = v[5];
        g = v[6];
        h = v[7];
        for (i = 0; i < 16; i++) {
            j = pos + i * 4;
            w[i] = (((p[j] & 0xff) << 24) | ((p[j + 1] & 0xff) << 16) |
                ((p[j + 2] & 0xff) << 8) | (p[j + 3] & 0xff));
        }
        for (i = 16; i < 64; i++) {
            u = w[i - 2];
            t1 = (u >>> 17 | u << (32 - 17)) ^ (u >>> 19 | u << (32 - 19)) ^ (u >>> 10);
            u = w[i - 15];
            t2 = (u >>> 7 | u << (32 - 7)) ^ (u >>> 18 | u << (32 - 18)) ^ (u >>> 3);
            w[i] = (t1 + w[i - 7] | 0) + (t2 + w[i - 16] | 0);
        }
        for (i = 0; i < 64; i++) {
            t1 = (((((e >>> 6 | e << (32 - 6)) ^ (e >>> 11 | e << (32 - 11)) ^
                (e >>> 25 | e << (32 - 25))) + ((e & f) ^ (~e & g))) | 0) +
                ((h + ((K$1[i] + w[i]) | 0)) | 0)) | 0;
            t2 = (((a >>> 2 | a << (32 - 2)) ^ (a >>> 13 | a << (32 - 13)) ^
                (a >>> 22 | a << (32 - 22))) + ((a & b) ^ (a & c) ^ (b & c))) | 0;
            h = g;
            g = f;
            f = e;
            e = (d + t1) | 0;
            d = c;
            c = b;
            b = a;
            a = (t1 + t2) | 0;
        }
        v[0] += a;
        v[1] += b;
        v[2] += c;
        v[3] += d;
        v[4] += e;
        v[5] += f;
        v[6] += g;
        v[7] += h;
        pos += 64;
        len -= 64;
    }
    return pos;
}
let Hash$2 = class Hash {
    constructor() {
        this.digestLength = digestLength$1;
        this.blockSize = blockSize$1;
        this.state = new Int32Array(8);
        this.temp = new Int32Array(64);
        this.buffer = new Uint8Array(128);
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        this.reset();
    }
    reset() {
        this.state[0] = 0x6a09e667;
        this.state[1] = 0xbb67ae85;
        this.state[2] = 0x3c6ef372;
        this.state[3] = 0xa54ff53a;
        this.state[4] = 0x510e527f;
        this.state[5] = 0x9b05688c;
        this.state[6] = 0x1f83d9ab;
        this.state[7] = 0x5be0cd19;
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        return this;
    }
    clean() {
        for (let i = 0; i < this.buffer.length; i++) {
            this.buffer[i] = 0;
        }
        for (let i = 0; i < this.temp.length; i++) {
            this.temp[i] = 0;
        }
        this.reset();
    }
    update(data, dataLength = data.length) {
        if (this.finished) {
            throw new Error("SHA256: can't update because hash was finished.");
        }
        let dataPos = 0;
        this.bytesHashed += dataLength;
        if (this.bufferLength > 0) {
            while (this.bufferLength < 64 && dataLength > 0) {
                this.buffer[this.bufferLength++] = data[dataPos++];
                dataLength--;
            }
            if (this.bufferLength === 64) {
                hashBlocks$1(this.temp, this.state, this.buffer, 0, 64);
                this.bufferLength = 0;
            }
        }
        if (dataLength >= 64) {
            dataPos = hashBlocks$1(this.temp, this.state, data, dataPos, dataLength);
            dataLength %= 64;
        }
        while (dataLength > 0) {
            this.buffer[this.bufferLength++] = data[dataPos++];
            dataLength--;
        }
        return this;
    }
    finish(out) {
        if (!this.finished) {
            const bytesHashed = this.bytesHashed;
            const left = this.bufferLength;
            const bitLenHi = (bytesHashed / 0x20000000) | 0;
            const bitLenLo = bytesHashed << 3;
            const padLength = (bytesHashed % 64 < 56) ? 64 : 128;
            this.buffer[left] = 0x80;
            for (let i = left + 1; i < padLength - 8; i++) {
                this.buffer[i] = 0;
            }
            this.buffer[padLength - 8] = (bitLenHi >>> 24) & 0xff;
            this.buffer[padLength - 7] = (bitLenHi >>> 16) & 0xff;
            this.buffer[padLength - 6] = (bitLenHi >>> 8) & 0xff;
            this.buffer[padLength - 5] = (bitLenHi >>> 0) & 0xff;
            this.buffer[padLength - 4] = (bitLenLo >>> 24) & 0xff;
            this.buffer[padLength - 3] = (bitLenLo >>> 16) & 0xff;
            this.buffer[padLength - 2] = (bitLenLo >>> 8) & 0xff;
            this.buffer[padLength - 1] = (bitLenLo >>> 0) & 0xff;
            hashBlocks$1(this.temp, this.state, this.buffer, 0, padLength);
            this.finished = true;
        }
        for (let i = 0; i < 8; i++) {
            out[i * 4 + 0] = (this.state[i] >>> 24) & 0xff;
            out[i * 4 + 1] = (this.state[i] >>> 16) & 0xff;
            out[i * 4 + 2] = (this.state[i] >>> 8) & 0xff;
            out[i * 4 + 3] = (this.state[i] >>> 0) & 0xff;
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
    _saveState(out) {
        for (let i = 0; i < this.state.length; i++) {
            out[i] = this.state[i];
        }
    }
    _restoreState(from, bytesHashed) {
        for (let i = 0; i < this.state.length; i++) {
            this.state[i] = from[i];
        }
        this.bytesHashed = bytesHashed;
        this.finished = false;
        this.bufferLength = 0;
    }
};
let HMAC$1 = class HMAC {
    constructor(key) {
        this.inner = new Hash$2();
        this.outer = new Hash$2();
        this.blockSize = this.inner.blockSize;
        this.digestLength = this.inner.digestLength;
        const pad = new Uint8Array(this.blockSize);
        if (key.length > this.blockSize) {
            (new Hash$2()).update(key).finish(pad).clean();
        }
        else {
            for (let i = 0; i < key.length; i++) {
                pad[i] = key[i];
            }
        }
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36;
        }
        this.inner.update(pad);
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36 ^ 0x5c;
        }
        this.outer.update(pad);
        this.istate = new Uint32Array(8);
        this.ostate = new Uint32Array(8);
        this.inner._saveState(this.istate);
        this.outer._saveState(this.ostate);
        for (let i = 0; i < pad.length; i++) {
            pad[i] = 0;
        }
    }
    reset() {
        this.inner._restoreState(this.istate, this.inner.blockSize);
        this.outer._restoreState(this.ostate, this.outer.blockSize);
        return this;
    }
    clean() {
        for (let i = 0; i < this.istate.length; i++) {
            this.ostate[i] = this.istate[i] = 0;
        }
        this.inner.clean();
        this.outer.clean();
    }
    update(data) {
        this.inner.update(data);
        return this;
    }
    finish(out) {
        if (this.outer.finished) {
            this.outer.finish(out);
        }
        else {
            this.inner.finish(out);
            this.outer.update(out, this.digestLength).finish(out);
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
};
function sha256$2(data) {
    const h = (new Hash$2()).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}
function hash256$2(data) {
    return sha256$2(sha256$2(data));
}
function hmac256$2(key, data) {
    const h = (new HMAC$1(key)).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}

function addChecksum$1(data) {
    const sum = hash256$2(data);
    return Uint8Array.of(...data, ...sum.slice(0, 4));
}
function checkTheSum$1(data) {
    const ret = data.slice(0, -4);
    const chk = data.slice(-4);
    const sum = hash256$2(ret).slice(0, 4);
    if (sum.toString() !== chk.toString()) {
        throw new Error('Invalid checksum!');
    }
    return ret;
}

const crypto$3 = globalThis.crypto;
let Buff$1 = class Buff extends Uint8Array {
    static { this.num = (number, size, endian = 'le') => {
        const b = new Buff$1(numToBytes$1(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.big = (number, size, endian = 'le') => {
        const b = new Buff$1(bigToBytes$1(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static b58check(data) {
        const decoded = BaseX$1.decode(data, 'base58');
        return new Buff$1(checkTheSum$1(decoded));
    }
    static { this.any = (data, size) => new Buff$1(buffer$1(data, false), size); }
    static { this.raw = (data, size) => new Buff$1(data, size); }
    static { this.str = (data, size) => new Buff$1(strToBytes$1(data), size); }
    static { this.hex = (data, size) => new Buff$1(hexToBytes$2(data), size); }
    static { this.bin = (data, size) => new Buff$1(binaryToBytes(data), size); }
    static { this.json = (data) => new Buff$1(strToBytes$1(JSON.stringify(data))); }
    static { this.bytes = (data, size) => new Buff$1(buffer$1(data, true), size); }
    static { this.base64 = (data) => new Buff$1(Base64.decode(data)); }
    static { this.b64url = (data) => new Buff$1(B64URL.decode(data)); }
    static { this.bech32 = (data, ver = 0) => new Buff$1(Bech32$1.decode(data, ver)); }
    static { this.bech32m = (data) => new Buff$1(Bech32$1.decode(data, 1)); }
    constructor(data, size) {
        data = buffer$1(data, true);
        if (typeof size === 'number') {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        return this;
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get arr() {
        return this.toArr();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get bin() {
        return this.toBin();
    }
    get base64() {
        return this.toBase64();
    }
    get b64url() {
        return this.toB64url();
    }
    get digest() {
        return this.toHash();
    }
    get id() {
        return this.toHash().hex;
    }
    get stream() {
        return new Stream(this);
    }
    toNum(endian = 'le') {
        return (endian === 'le')
            ? bytesToNum$1(this.reverse())
            : bytesToNum$1(this);
    }
    toBig(endian = 'le') {
        return (endian === 'le')
            ? bytesToBig$1(this.reverse())
            : bytesToBig$1(this);
    }
    toHash(type = 'sha256') {
        switch (type) {
            case 'sha256':
                return new Buff$1(sha256$2(this));
            case 'hash256':
                return new Buff$1(sha256$2(sha256$2(this)));
            case 'ripe160':
                return new Buff$1(ripemd160$1(this));
            case 'hash160':
                return new Buff$1(ripemd160$1(sha256$2(this)));
            default:
                throw new Error('Unrecognized format:' + String(type));
        }
    }
    toHmac(key) {
        if (typeof key === 'string') {
            key = strToBytes$1(key);
        }
        return new Buff$1(hmac256$2(key, this));
    }
    tob58check() {
        return BaseX$1.encode(addChecksum$1(this), 'base58');
    }
    toArr() { return Array.from(this); }
    toStr() { return bytesToStr$1(this); }
    toHex() { return bytesToHex$2(this); }
    toJson() { return JSON.parse(bytesToStr$1(this)); }
    toBytes() { return new Uint8Array(this); }
    toBin() { return bytesToBinary(this); }
    toB64url() { return B64URL.encode(this); }
    toBase64() { return Base64.encode(this); }
    toBech32(hrp, ver = 0) { return Bech32$1.encode(this, hrp, ver); }
    toBech32m(hrp) { return Bech32$1.encode(this, hrp, 1); }
    prepend(data) {
        return Buff$1.join([Buff$1.bytes(data), this]);
    }
    append(data) {
        return Buff$1.join([this, Buff$1.bytes(data)]);
    }
    slice(start, end) {
        return new Buff$1(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff$1(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prefixSize(endian) {
        const size = Buff$1.varInt(this.length, endian);
        return Buff$1.join([size, this]);
    }
    static from(data) {
        return new Buff$1(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff$1(Uint8Array.of(...args));
    }
    static join(arr) {
        let i, idx = 0;
        const data = arr.map(e => Buff$1.bytes(e));
        const size = data.reduce((prev, curr) => prev + curr.length, 0);
        const buff = new Buff$1(0, size);
        for (const bytes of data) {
            for (i = 0; i < bytes.length; idx++, i++) {
                buff[idx] = bytes[i];
            }
        }
        return buff;
    }
    static varInt(num, endian) {
        if (num < 0xFD) {
            return Buff$1.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff$1.of(0xFD, ...Buff$1.num(num, 2, endian));
        }
        else if (num < 0x100000000) {
            return Buff$1.of(0xFE, ...Buff$1.num(num, 4, endian));
        }
        else if (BigInt(num) < 0x10000000000000000n) {
            return Buff$1.of(0xFF, ...Buff$1.num(num, 8, endian));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static random(size = 32) {
        return new Buff$1(crypto$3.getRandomValues(new Uint8Array(size)));
    }
    static { this.encode = strToBytes$1; }
    static { this.decode = bytesToStr$1; }
    static normalize(data, size) {
        if (data instanceof Uint8Array)
            return data;
        if (typeof data === 'string')
            return Buff$1.hex(data, size).raw;
        if (typeof data === 'number')
            return Buff$1.num(data, size).raw;
        if (typeof data === 'bigint')
            return Buff$1.big(data, size).raw;
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static serialize(data) {
        if (typeof data === 'string') {
            return Buff$1.str(data).raw;
        }
        if (typeof data === 'object') {
            if (data instanceof Uint8Array) {
                return data;
            }
            try {
                return Buff$1.json(data).raw;
            }
            catch {
                throw TypeError('Object is not serializable.');
            }
        }
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static revitalize(data) {
        if (data instanceof Uint8Array) {
            data = bytesToStr$1(data);
        }
        if (typeof data === 'string') {
            try {
                return JSON.parse(data);
            }
            catch {
                return data;
            }
        }
        return data;
    }
};
class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        return new Buff$1(this.data.slice(0, size).buffer);
    }
    read(size) {
        size = size ?? this.readSize();
        const chunk = this.peek(size);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    readSize() {
        const num = this.read(1).toNum();
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum();
            case (num === 0xFE):
                return this.read(4).toNum();
            case (num === 0xFF):
                return this.read(8).toNum();
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
}

const OPCODE_MAP = {
    OP_0: 0,
    OP_PUSHDATA1: 76,
    OP_PUSHDATA2: 77,
    OP_PUSHDATA4: 78,
    OP_1NEGATE: 79,
    OP_SUCCESS80: 80,
    OP_1: 81,
    OP_2: 82,
    OP_3: 83,
    OP_4: 84,
    OP_5: 85,
    OP_6: 86,
    OP_7: 87,
    OP_8: 88,
    OP_9: 89,
    OP_10: 90,
    OP_11: 91,
    OP_12: 92,
    OP_13: 93,
    OP_14: 94,
    OP_15: 95,
    OP_16: 96,
    OP_NOP: 97,
    OP_SUCCESS98: 98,
    OP_IF: 99,
    OP_NOTIF: 100,
    OP_ELSE: 103,
    OP_ENDIF: 104,
    OP_VERIFY: 105,
    OP_RETURN: 106,
    OP_TOALTSTACK: 107,
    OP_FROMALTSTACK: 108,
    OP_2DROP: 109,
    OP_2DUP: 110,
    OP_3DUP: 111,
    OP_2OVER: 112,
    OP_2ROT: 113,
    OP_2SWAP: 114,
    OP_IFDUP: 115,
    OP_DEPTH: 116,
    OP_DROP: 117,
    OP_DUP: 118,
    OP_NIP: 119,
    OP_OVER: 120,
    OP_PICK: 121,
    OP_ROLL: 122,
    OP_ROT: 123,
    OP_SWAP: 124,
    OP_TUCK: 125,
    OP_SUCCESS126: 126,
    OP_SUCCESS127: 127,
    OP_SUCCESS128: 128,
    OP_SUCCESS129: 129,
    OP_SIZE: 130,
    OP_SUCCESS131: 131,
    OP_SUCCESS132: 132,
    OP_SUCCESS133: 133,
    OP_SUCCESS134: 134,
    OP_EQUAL: 135,
    OP_EQUALVERIFY: 136,
    OP_SUCCESS137: 137,
    OP_SUCCESS138: 138,
    OP_1ADD: 139,
    OP_1SUB: 140,
    OP_SUCCESS141: 141,
    OP_SUCCESS142: 142,
    OP_NEGATE: 143,
    OP_ABS: 144,
    OP_NOT: 145,
    OP_0NOTEQUAL: 146,
    OP_ADD: 147,
    OP_SUB: 148,
    OP_SUCCESS149: 149,
    OP_SUCCESS150: 150,
    OP_SUCCESS151: 151,
    OP_SUCCESS152: 152,
    OP_SUCCESS153: 153,
    OP_BOOLAND: 154,
    OP_BOOLOR: 155,
    OP_NUMEQUAL: 156,
    OP_NUMEQUALVERIFY: 157,
    OP_NUMNOTEQUAL: 158,
    OP_LESSTHAN: 159,
    OP_GREATERTHAN: 160,
    OP_LESSTHANOREQUAL: 161,
    OP_GREATERTHANOREQUAL: 162,
    OP_MIN: 163,
    OP_MAX: 164,
    OP_WITHIN: 165,
    OP_RIPEMD160: 166,
    OP_SHA1: 167,
    OP_SHA256: 168,
    OP_HASH160: 169,
    OP_HASH256: 170,
    OP_CODESEPARATOR: 171,
    OP_CHECKSIG: 172,
    OP_CHECKSIGVERIFY: 173,
    OP_CHECKMULTISIG: 174,
    OP_CHECKMULTISIGVERIFY: 175,
    OP_NOP1: 176,
    OP_CHECKLOCKTIMEVERIFY: 177,
    OP_CHECKSEQUENCEVERIFY: 178,
    OP_NOP4: 179,
    OP_NOP5: 180,
    OP_NOP6: 181,
    OP_NOP7: 182,
    OP_NOP8: 183,
    OP_NOP9: 184,
    OP_NOP10: 185,
    OP_CHECKSIGADD: 186
};
function getOpLabel(num) {
    if (num > 186 && num < 255) {
        return 'OP_SUCCESS' + String(num);
    }
    for (const [k, v] of Object.entries(OPCODE_MAP)) {
        if (v === num)
            return k;
    }
    throw new Error('OPCODE not found:' + String(num));
}
function getOpCode(string) {
    for (const [k, v] of Object.entries(OPCODE_MAP)) {
        if (k === string)
            return Number(v);
    }
    throw new Error('OPCODE not found:' + string);
}
function getWordType(word) {
    switch (true) {
        case (word === 0):
            return 'opcode';
        case (word >= 1 && word <= 75):
            return 'varint';
        case (word === 76):
            return 'pushdata1';
        case (word === 77):
            return 'pushdata2';
        case (word === 78):
            return 'pushdata4';
        case (word <= 254):
            return 'opcode';
        default:
            throw new Error(`Invalid word range: ${word}`);
    }
}
function isValidWord(word) {
    const MIN_RANGE = 75;
    const MAX_RANGE = 254;
    const DISABLED_OPCODES = [];
    switch (true) {
        case (typeof (word) !== 'number'):
            return false;
        case (word === 0):
            return true;
        case (DISABLED_OPCODES.includes(word)):
            return false;
        case (MIN_RANGE < word && word < MAX_RANGE):
            return true;
        default:
            return false;
    }
}

const MAX_WORD_SIZE = 0x208;
function encodeScript(script = [], varint = true) {
    let buff = Buff$1.num(0);
    if (Array.isArray(script)) {
        buff = Buff$1.raw(encodeWords(script));
    }
    if (typeof script === 'string') {
        buff = Buff$1.hex(script);
    }
    if (script instanceof Uint8Array) {
        buff = Buff$1.raw(script);
    }
    if (varint) {
        buff = buff.prefixSize('be');
    }
    return buff.raw;
}
function encodeWords(wordArray) {
    const words = [];
    for (const word of wordArray) {
        words.push(encodeWord(word));
    }
    return (words.length > 0)
        ? Buff$1.join(words)
        : new Uint8Array();
}
function encodeWord(word) {
    let buff = new Uint8Array();
    if (typeof (word) === 'string') {
        if (word.startsWith('OP_')) {
            return Buff$1.num(getOpCode(word), 1);
        }
        if (word.startsWith('STR_')) {
            buff = Buff$1.str(word.slice(3));
        }
        buff = Buff$1.hex(word);
    }
    if (typeof (word) === 'number') {
        buff = Buff$1.num(word);
    }
    if (word instanceof Uint8Array) {
        buff = word;
    }
    if (buff.length > MAX_WORD_SIZE) {
        const words = splitWord(buff);
        return encodeWords(words);
    }
    return Buff$1.join([encodeSize(buff.length), buff]);
}
function encodeSize(size) {
    const OP_DATAPUSH1 = Buff$1.num(0x4c, 1);
    const OP_DATAPUSH2 = Buff$1.num(0x4d, 1);
    switch (true) {
        case (size <= 0x4b):
            return Buff$1.num(size);
        case (size > 0x4b && size < 0x100):
            return Buff$1.join([OP_DATAPUSH1, Buff$1.num(size, 1)]);
        case (size >= 0x100 && size <= MAX_WORD_SIZE):
            return Buff$1.join([OP_DATAPUSH2, Buff$1.num(size, 2, 'be')]);
        default:
            throw new Error('Invalid word size:' + size.toString());
    }
}
function splitWord(word) {
    const words = [];
    const buff = new Stream(word);
    while (buff.size > MAX_WORD_SIZE) {
        words.push(buff.read(MAX_WORD_SIZE));
    }
    words.push(buff.read(buff.size));
    return words;
}

function check$3(address, network = 'main') {
    const prefixes = (network === 'main') ? ['1'] : ['m', 'n'];
    for (const prefix of prefixes) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$5(key, network = 'main') {
    const prefix = (network === 'main') ? Buff$1.num(0x00) : Buff$1.num(0x6F);
    const bytes = Buff$1.bytes(key).prepend(prefix);
    return bytes.toHash('hash160').tob58check();
}
function decode$5(address, network = 'main') {
    if (!check$3(address, network)) {
        throw new TypeError('Invalid p2pkh address!');
    }
    return Buff$1.b58check(address);
}
function script$3(key) {
    return ['OP_DUP', 'OP_HASH160', key, 'OP_EQUALVERIFY', 'OP_CHECKSIG'];
}
const P2PKH = { check: check$3, encode: encode$5, decode: decode$5, script: script$3 };

function check$2(address, network = 'main') {
    const prefixes = (network === 'main') ? ['3'] : ['2'];
    for (const prefix of prefixes) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$4(key, network = 'main') {
    const prefix = (network === 'main') ? Buff$1.num(0x05) : Buff$1.num(0xC4);
    const bytes = Buff$1.bytes(key).prepend(prefix);
    return bytes.toHash('hash160').tob58check();
}
function decode$4(address, network = 'main') {
    if (!check$2(address, network)) {
        throw new TypeError('Invalid p2sh address!');
    }
    return Buff$1.b58check(address);
}
function script$2(key) {
    return ['OP_HASH160', key, 'OP_EQUAL'];
}
const P2SH = { check: check$2, encode: encode$4, decode: decode$4, script: script$2 };

const VALID_PREFIXES$2 = {
    '1': ['p2pkh', 'main'],
    '3': ['p2sh', 'main'],
    'm': ['p2pkh', 'testnet'],
    'n': ['p2pkh', 'testnet'],
    '2': ['p2sh', 'testnet'],
    'bc1q': ['p2w', 'main'],
    'tb1q': ['p2w', 'testnet'],
    'bcrt1q': ['p2w', 'regtest'],
    'bc1p': ['p2tr', 'main'],
    'tb1p': ['p2tr', 'testnet'],
    'bcrt1p': ['p2tr', 'regtest']
};
const BECH32_PREFIXES = {
    main: 'bc',
    testnet: 'tb',
    signet: 'tb',
    regtest: 'bcrt'
};

const VALID_PREFIXES$1 = ['bc1q', 'tb1q', 'bcrt1q'];
function check$1(address) {
    for (const prefix of VALID_PREFIXES$1) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$3(key, network = 'main') {
    const bytes = Buff$1.bytes(key);
    const prefix = BECH32_PREFIXES[network];
    if (bytes.length !== 20 && bytes.length !== 32) {
        throw new Error('Key length is an invalid size: ' + String(bytes.length));
    }
    const hash = (bytes.length === 20)
        ? bytes.toHash('hash160')
        : bytes.toHash('sha256');
    return hash.toBech32(prefix, 0);
}
function decode$3(address) {
    if (!check$1(address)) {
        throw new TypeError('Invalid segwit address!');
    }
    return Buff$1.bech32(address, 0);
}
function script$1(key) {
    return ['00', key];
}
const P2W = { check: check$1, encode: encode$3, decode: decode$3, script: script$1 };

const VALID_PREFIXES = ['bc1p', 'tb1p', 'bcrt1p'];
function check(address) {
    for (const prefix of VALID_PREFIXES) {
        if (address.startsWith(prefix)) {
            return true;
        }
    }
    return false;
}
function encode$2(key, network = 'main') {
    const bytes = Buff$1.bytes(key);
    const prefix = BECH32_PREFIXES[network];
    if (bytes.length !== 32) {
        throw new Error('Key length is an invalid size: ' + String(bytes.length));
    }
    return bytes.toBech32(prefix, 1);
}
function decode$2(address) {
    if (!check(address)) {
        throw new TypeError('Invalid taproot address!');
    }
    return Buff$1.bech32(address, 1);
}
function script(key) {
    return ['51', key];
}
const P2TR = { check, encode: encode$2, decode: decode$2, script };

const ADDRESS_TOOLS = {
    'p2pkh': P2PKH,
    'p2sh': P2SH,
    'p2w': P2W,
    'p2tr': P2TR
};
function getType(address) {
    for (const key of Object.keys(VALID_PREFIXES$2)) {
        if (address.startsWith(key)) {
            return VALID_PREFIXES$2[key];
        }
    }
    throw new Error('Unable to detect address type!');
}
function convert(address) {
    const [type, network] = getType(address);
    const tool = ADDRESS_TOOLS[type];
    const bytes = tool.decode(address, network).hex;
    return tool.script(bytes);
}

const Address = {
    P2PKH,
    P2SH,
    P2W,
    P2TR,
    getType,
    convert
};

function encodeTx(txdata, omitWitness) {
    const { version, input, output, locktime } = txdata;
    const useWitness = (omitWitness !== true && checkForWitness(input));
    const raw = [encodeVersion(version)];
    if (useWitness) {
        raw.push(Buff$1.hex('0001'));
    }
    raw.push(encodeInputs(input));
    raw.push(encodeOutputs(output));
    for (const vin of input) {
        if (useWitness && vin?.witness !== undefined) {
            raw.push(encodeWitness(vin.witness));
        }
    }
    raw.push(encodeLocktime(locktime));
    return Buff$1.join(raw).hex;
}
function checkForWitness(vin) {
    for (const txin of vin) {
        const { witness } = txin;
        if (typeof witness === 'string' ||
            witness instanceof Uint8Array ||
            (Array.isArray(witness) && witness.length > 0)) {
            return true;
        }
    }
    return false;
}
function encodeVersion(num) {
    return Buff$1.num(num, 4).reverse();
}
function encodeTxid(txid) {
    return Buff$1.hex(txid, 32).reverse();
}
function encodePrevOut(vout) {
    return Buff$1.num(vout, 4).reverse();
}
function encodeSequence(seq = 0xFFFFFFFF) {
    const sequence = (typeof seq === 'string')
        ? Buff$1.hex(seq, 4)
        : Buff$1.num(seq, 4);
    return sequence.reverse();
}
function encodeInputs(arr) {
    const raw = [Buff$1.varInt(arr.length)];
    for (const vin of arr) {
        const { txid, vout, scriptSig, sequence } = vin;
        raw.push(encodeTxid(txid));
        raw.push(encodePrevOut(vout));
        raw.push(encodeScript(scriptSig));
        raw.push(encodeSequence(sequence));
    }
    return Buff$1.join(raw);
}
function encodeValue(value) {
    if (typeof value === 'number') {
        value = BigInt(value);
    }
    return Buff$1.big(value, 8).reverse();
}
function encodeOutputs(arr) {
    const raw = [Buff$1.varInt(arr.length)];
    for (const vout of arr) {
        raw.push(encodeOutput(vout));
    }
    return Buff$1.join(raw);
}
function encodeOutput(vout) {
    const raw = [];
    const { address, value, scriptPubKey } = vout;
    raw.push(encodeValue(value));
    if (typeof address === 'string') {
        const script = Address.convert(address);
        raw.push(encodeScript(script));
    }
    else {
        raw.push(encodeScript(scriptPubKey));
    }
    return Buff$1.join(raw);
}
function encodeWitness(data) {
    const buffer = [];
    if (Array.isArray(data)) {
        buffer.push(Buff$1.varInt(data.length));
        for (const entry of data) {
            buffer.push(encodeScript(entry));
        }
        return Buff$1.join(buffer);
    }
    else {
        return Buff$1.normalize(data);
    }
}
function encodeLocktime(num) {
    return Buff$1.num(num, 4).reverse();
}

function decodeTx(bytes) {
    if (typeof bytes === 'string') {
        bytes = Buff$1.hex(bytes).raw;
    }
    const stream = new Stream(bytes);
    const version = readVersion(stream);
    const hasWitness = checkWitnessFlag(stream);
    const input = readInputs(stream);
    const output = readOutputs(stream);
    if (hasWitness) {
        for (const vin of input) {
            vin.witness = readWitness(stream);
        }
    }
    const locktime = readLocktime(stream);
    return { version, input, output, locktime };
}
function readVersion(stream) {
    return stream.read(4).reverse().toNum();
}
function checkWitnessFlag(stream) {
    const [marker, flag] = [...stream.peek(2)];
    if (marker === 0) {
        stream.read(2);
        if (flag === 1) {
            return true;
        }
        else {
            throw new Error(`Invalid witness flag: ${flag}`);
        }
    }
    return false;
}
function readInputs(stream) {
    const inputs = [];
    const vinCount = stream.readSize();
    for (let i = 0; i < vinCount; i++) {
        inputs.push(readInput(stream));
    }
    return inputs;
}
function readInput(stream) {
    return {
        txid: stream.read(32).reverse().toHex(),
        vout: stream.read(4).reverse().toNum(),
        scriptSig: readScript(stream, true),
        sequence: stream.read(4).reverse().toHex()
    };
}
function readOutputs(stream) {
    const outputs = [];
    const outcount = stream.readSize();
    for (let i = 0; i < outcount; i++) {
        outputs.push(readOutput(stream));
    }
    return outputs;
}
function readOutput(stream) {
    return {
        value: stream.read(8).reverse().big,
        scriptPubKey: readScript(stream, true)
    };
}
function readWitness(stream) {
    const stack = [];
    const count = stream.readSize();
    for (let i = 0; i < count; i++) {
        const word = readData(stream, true);
        stack.push(word);
    }
    return stack;
}
function readData(stream, hasVarint) {
    const size = (hasVarint === true)
        ? stream.readSize()
        : stream.size;
    return size > 0
        ? stream.read(size).toHex()
        : Buff$1.num(0).toHex();
}
function readScript(stream, hasVarint) {
    const data = readData(stream, hasVarint);
    return (data !== '00') ? data : [];
}
function readLocktime(stream) {
    return stream.read(4).reverse().toNum();
}
function normalizeTx(txdata) {
    if (typeof txdata === 'string' ||
        txdata instanceof Uint8Array) {
        txdata = decodeTx(txdata);
    }
    return { ...txdata };
}

function decodeScript(script) {
    return decodeWords(Buff$1.normalize(script));
}
function normalizeData(script) {
    if (script === undefined) {
        throw new Error('Script data is undefined!');
    }
    if (Array.isArray(script)) {
        throw new Error('Script data is an array!');
    }
    return Buff$1.normalize(script);
}
function decodeWords(words, fmt = 'asm') {
    const stream = new Stream(words);
    const stack = [];
    const stackSize = stream.size;
    let word;
    let wordType;
    let wordSize;
    let count = 0;
    while (count < stackSize) {
        word = stream.read(1).num;
        wordType = getWordType(word);
        count++;
        switch (wordType) {
            case 'varint':
                stack.push(stream.read(word).toHex());
                count += word;
                break;
            case 'pushdata1':
                wordSize = stream.read(1).reverse().num;
                stack.push(stream.read(wordSize).toHex());
                count += wordSize + 1;
                break;
            case 'pushdata2':
                wordSize = stream.read(2).reverse().num;
                stack.push(stream.read(wordSize).toHex());
                count += wordSize + 2;
                break;
            case 'pushdata4':
                wordSize = stream.read(4).reverse().num;
                stack.push(stream.read(wordSize).toHex());
                count += wordSize + 4;
                break;
            case 'opcode':
                if (!isValidWord(word)) {
                    throw new Error(`Invalid OPCODE: ${word}`);
                }
                if (fmt === 'asm') {
                    stack.push(getOpLabel(word));
                }
                else {
                    stack.push(word);
                }
                break;
            default:
                throw new Error(`Word type undefined: ${word}`);
        }
    }
    return stack;
}

/*! noble-secp256k1 - MIT License (c) 2019 Paul Miller (paulmillr.com) */
const crypto$2 = getCryptoLib$1();
function getCryptoLib$1() {
    if (typeof globalThis !== 'undefined') {
        return globalThis.crypto;
    }
    if (typeof window !== 'undefined') {
        return window.crypto;
    }
    throw new Error('Must be using https in browser, or node/deno environment that supports globalthis.crypto object!');
}
const _0n = BigInt(0);
const _1n = BigInt(1);
const _2n = BigInt(2);
const _3n = BigInt(3);
const _8n = BigInt(8);
const CURVE = Object.freeze({
    a: _0n,
    b: BigInt(7),
    P: BigInt('0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f'),
    n: BigInt('0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141'),
    h: _1n,
    Gx: BigInt('55066263022277343669578718895168534326250603453777594175500187360389116729240'),
    Gy: BigInt('32670510020758816978083085130507043184471273380659243275938904335757337482424'),
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee'),
});
const divNearest = (a, b) => (a + b / _2n) / b;
const endo = {
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee'),
    splitScalar(k) {
        const { n } = CURVE;
        const a1 = BigInt('0x3086d221a7d46bcde86c90e49284eb15');
        const b1 = -_1n * BigInt('0xe4437ed6010e88286f547fa90abfe4c3');
        const a2 = BigInt('0x114ca50f7a8e2f3f657c1108d9d44cfd8');
        const b2 = a1;
        const POW_2_128 = BigInt('0x100000000000000000000000000000000');
        const c1 = divNearest(b2 * k, n);
        const c2 = divNearest(-b1 * k, n);
        let k1 = mod(k - c1 * a1 - c2 * a2, n);
        let k2 = mod(-c1 * b1 - c2 * b2, n);
        const k1neg = k1 > POW_2_128;
        const k2neg = k2 > POW_2_128;
        if (k1neg)
            k1 = n - k1;
        if (k2neg)
            k2 = n - k2;
        if (k1 > POW_2_128 || k2 > POW_2_128) {
            throw new Error('splitScalarEndo: Endomorphism failed, k=' + k);
        }
        return { k1neg, k1, k2neg, k2 };
    },
};
const fieldLen = 32;
const groupLen = 32;
const compressedLen = fieldLen + 1;
const uncompressedLen = 2 * fieldLen + 1;
function weierstrass(x) {
    const { a, b } = CURVE;
    const x2 = mod(x * x);
    const x3 = mod(x2 * x);
    return mod(x3 + a * x + b);
}
const USE_ENDOMORPHISM = CURVE.a === _0n;
class ShaError extends Error {
    constructor(message) {
        super(message);
    }
}
function assertJacPoint(other) {
    if (!(other instanceof JacobianPoint))
        throw new TypeError('JacobianPoint expected');
}
class JacobianPoint {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }
    static get BASE() {
        return new JacobianPoint(CURVE.Gx, CURVE.Gy, _1n);
    }
    static get ZERO() {
        return new JacobianPoint(_0n, _1n, _0n);
    }
    static fromAffine(p) {
        if (!(p instanceof Point$1)) {
            throw new TypeError('JacobianPoint#fromAffine: expected Point');
        }
        if (p.equals(Point$1.ZERO))
            return JacobianPoint.ZERO;
        return new JacobianPoint(p.x, p.y, _1n);
    }
    static toAffineBatch(points) {
        const toInv = invertBatch(points.map((p) => p.z));
        return points.map((p, i) => p.toAffine(toInv[i]));
    }
    static normalizeZ(points) {
        return JacobianPoint.toAffineBatch(points).map(JacobianPoint.fromAffine);
    }
    equals(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        return U1 === U2 && S1 === S2;
    }
    negate() {
        return new JacobianPoint(this.x, mod(-this.y), this.z);
    }
    double() {
        const { x: X1, y: Y1, z: Z1 } = this;
        const A = mod(X1 * X1);
        const B = mod(Y1 * Y1);
        const C = mod(B * B);
        const x1b = X1 + B;
        const D = mod(_2n * (mod(x1b * x1b) - A - C));
        const E = mod(_3n * A);
        const F = mod(E * E);
        const X3 = mod(F - _2n * D);
        const Y3 = mod(E * (D - X3) - _8n * C);
        const Z3 = mod(_2n * Y1 * Z1);
        return new JacobianPoint(X3, Y3, Z3);
    }
    add(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        if (X2 === _0n || Y2 === _0n)
            return this;
        if (X1 === _0n || Y1 === _0n)
            return other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        const H = mod(U2 - U1);
        const r = mod(S2 - S1);
        if (H === _0n) {
            if (r === _0n) {
                return this.double();
            }
            else {
                return JacobianPoint.ZERO;
            }
        }
        const HH = mod(H * H);
        const HHH = mod(H * HH);
        const V = mod(U1 * HH);
        const X3 = mod(r * r - HHH - _2n * V);
        const Y3 = mod(r * (V - X3) - S1 * HHH);
        const Z3 = mod(Z1 * Z2 * H);
        return new JacobianPoint(X3, Y3, Z3);
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiplyUnsafe(scalar) {
        const P0 = JacobianPoint.ZERO;
        if (typeof scalar === 'bigint' && scalar === _0n)
            return P0;
        let n = normalizeScalar(scalar);
        if (n === _1n)
            return this;
        if (!USE_ENDOMORPHISM) {
            let p = P0;
            let d = this;
            while (n > _0n) {
                if (n & _1n)
                    p = p.add(d);
                d = d.double();
                n >>= _1n;
            }
            return p;
        }
        let { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
        let k1p = P0;
        let k2p = P0;
        let d = this;
        while (k1 > _0n || k2 > _0n) {
            if (k1 & _1n)
                k1p = k1p.add(d);
            if (k2 & _1n)
                k2p = k2p.add(d);
            d = d.double();
            k1 >>= _1n;
            k2 >>= _1n;
        }
        if (k1neg)
            k1p = k1p.negate();
        if (k2neg)
            k2p = k2p.negate();
        k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
        return k1p.add(k2p);
    }
    precomputeWindow(W) {
        const windows = USE_ENDOMORPHISM ? 128 / W + 1 : 256 / W + 1;
        const points = [];
        let p = this;
        let base = p;
        for (let window = 0; window < windows; window++) {
            base = p;
            points.push(base);
            for (let i = 1; i < 2 ** (W - 1); i++) {
                base = base.add(p);
                points.push(base);
            }
            p = base.double();
        }
        return points;
    }
    wNAF(n, affinePoint) {
        if (!affinePoint && this.equals(JacobianPoint.BASE))
            affinePoint = Point$1.BASE;
        const W = (affinePoint && affinePoint._WINDOW_SIZE) || 1;
        if (256 % W) {
            throw new Error('Point#wNAF: Invalid precomputation window, must be power of 2');
        }
        let precomputes = affinePoint && pointPrecomputes.get(affinePoint);
        if (!precomputes) {
            precomputes = this.precomputeWindow(W);
            if (affinePoint && W !== 1) {
                precomputes = JacobianPoint.normalizeZ(precomputes);
                pointPrecomputes.set(affinePoint, precomputes);
            }
        }
        let p = JacobianPoint.ZERO;
        let f = JacobianPoint.BASE;
        const windows = 1 + (USE_ENDOMORPHISM ? 128 / W : 256 / W);
        const windowSize = 2 ** (W - 1);
        const mask = BigInt(2 ** W - 1);
        const maxNumber = 2 ** W;
        const shiftBy = BigInt(W);
        for (let window = 0; window < windows; window++) {
            const offset = window * windowSize;
            let wbits = Number(n & mask);
            n >>= shiftBy;
            if (wbits > windowSize) {
                wbits -= maxNumber;
                n += _1n;
            }
            const offset1 = offset;
            const offset2 = offset + Math.abs(wbits) - 1;
            const cond1 = window % 2 !== 0;
            const cond2 = wbits < 0;
            if (wbits === 0) {
                f = f.add(constTimeNegate(cond1, precomputes[offset1]));
            }
            else {
                p = p.add(constTimeNegate(cond2, precomputes[offset2]));
            }
        }
        return { p, f };
    }
    multiply(scalar, affinePoint) {
        let n = normalizeScalar(scalar);
        let point;
        let fake;
        if (USE_ENDOMORPHISM) {
            const { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
            let { p: k1p, f: f1p } = this.wNAF(k1, affinePoint);
            let { p: k2p, f: f2p } = this.wNAF(k2, affinePoint);
            k1p = constTimeNegate(k1neg, k1p);
            k2p = constTimeNegate(k2neg, k2p);
            k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
            point = k1p.add(k2p);
            fake = f1p.add(f2p);
        }
        else {
            const { p, f } = this.wNAF(n, affinePoint);
            point = p;
            fake = f;
        }
        return JacobianPoint.normalizeZ([point, fake])[0];
    }
    toAffine(invZ) {
        const { x, y, z } = this;
        const is0 = this.equals(JacobianPoint.ZERO);
        if (invZ == null)
            invZ = is0 ? _8n : invert(z);
        const iz1 = invZ;
        const iz2 = mod(iz1 * iz1);
        const iz3 = mod(iz2 * iz1);
        const ax = mod(x * iz2);
        const ay = mod(y * iz3);
        const zz = mod(z * iz1);
        if (is0)
            return Point$1.ZERO;
        if (zz !== _1n)
            throw new Error('invZ was invalid');
        return new Point$1(ax, ay);
    }
}
function constTimeNegate(condition, item) {
    const neg = item.negate();
    return condition ? neg : item;
}
const pointPrecomputes = new WeakMap();
let Point$1 = class Point {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    _setWindowSize(windowSize) {
        this._WINDOW_SIZE = windowSize;
        pointPrecomputes.delete(this);
    }
    static get BASE() {
        return new Point$1(CURVE.Gx, CURVE.Gy);
    }
    static get ZERO() {
        return new Point$1(_0n, _0n);
    }
    hasEvenY() {
        return this.y % _2n === _0n;
    }
    static fromCompressedHex(bytes) {
        const isShort = bytes.length === 32;
        const x = bytesToNumber(isShort ? bytes : bytes.subarray(1));
        if (!isValidFieldElement(x))
            throw new Error('Point is not on curve');
        const y2 = weierstrass(x);
        let y = sqrtMod(y2);
        const isYOdd = (y & _1n) === _1n;
        if (isShort) {
            if (isYOdd)
                y = mod(-y);
        }
        else {
            const isFirstByteOdd = (bytes[0] & 1) === 1;
            if (isFirstByteOdd !== isYOdd)
                y = mod(-y);
        }
        const point = new Point$1(x, y);
        point.assertValidity();
        return point;
    }
    static fromUncompressedHex(bytes) {
        const x = bytesToNumber(bytes.subarray(1, fieldLen + 1));
        const y = bytesToNumber(bytes.subarray(fieldLen + 1, fieldLen * 2 + 1));
        const point = new Point$1(x, y);
        point.assertValidity();
        return point;
    }
    static fromHex(hex) {
        const bytes = ensureBytes(hex);
        const len = bytes.length;
        const header = bytes[0];
        if (len === fieldLen)
            return this.fromCompressedHex(bytes);
        if (len === compressedLen && (header === 0x02 || header === 0x03)) {
            return this.fromCompressedHex(bytes);
        }
        if (len === uncompressedLen && header === 0x04)
            return this.fromUncompressedHex(bytes);
        throw new Error(`Point.fromHex: received invalid point. Expected 32-${compressedLen} compressed bytes or ${uncompressedLen} uncompressed bytes, not ${len}`);
    }
    static fromPrivateKey(privateKey) {
        return Point$1.BASE.multiply(normalizePrivateKey(privateKey));
    }
    static fromSignature(msgHash, signature, recovery) {
        const { r, s } = normalizeSignature(signature);
        if (![0, 1, 2, 3].includes(recovery))
            throw new Error('Cannot recover: invalid recovery bit');
        const h = truncateHash(ensureBytes(msgHash));
        const { n } = CURVE;
        const radj = recovery === 2 || recovery === 3 ? r + n : r;
        const rinv = invert(radj, n);
        const u1 = mod(-h * rinv, n);
        const u2 = mod(s * rinv, n);
        const prefix = recovery & 1 ? '03' : '02';
        const R = Point$1.fromHex(prefix + numTo32bStr(radj));
        const Q = Point$1.BASE.multiplyAndAddUnsafe(R, u1, u2);
        if (!Q)
            throw new Error('Cannot recover signature: point at infinify');
        Q.assertValidity();
        return Q;
    }
    toRawBytes(isCompressed = false) {
        return hexToBytes$1(this.toHex(isCompressed));
    }
    toHex(isCompressed = false) {
        const x = numTo32bStr(this.x);
        if (isCompressed) {
            const prefix = this.hasEvenY() ? '02' : '03';
            return `${prefix}${x}`;
        }
        else {
            return `04${x}${numTo32bStr(this.y)}`;
        }
    }
    toHexX() {
        return this.toHex(true).slice(2);
    }
    toRawX() {
        return this.toRawBytes(true).slice(1);
    }
    assertValidity() {
        const msg = 'Point is not on elliptic curve';
        const { x, y } = this;
        if (!isValidFieldElement(x) || !isValidFieldElement(y))
            throw new Error(msg);
        const left = mod(y * y);
        const right = weierstrass(x);
        if (mod(left - right) !== _0n)
            throw new Error(msg);
    }
    equals(other) {
        return this.x === other.x && this.y === other.y;
    }
    negate() {
        return new Point$1(this.x, mod(-this.y));
    }
    double() {
        return JacobianPoint.fromAffine(this).double().toAffine();
    }
    add(other) {
        return JacobianPoint.fromAffine(this).add(JacobianPoint.fromAffine(other)).toAffine();
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiply(scalar) {
        return JacobianPoint.fromAffine(this).multiply(scalar, this).toAffine();
    }
    multiplyAndAddUnsafe(Q, a, b) {
        const P = JacobianPoint.fromAffine(this);
        const aP = a === _0n || a === _1n || this !== Point$1.BASE ? P.multiplyUnsafe(a) : P.multiply(a);
        const bQ = JacobianPoint.fromAffine(Q).multiplyUnsafe(b);
        const sum = aP.add(bQ);
        return sum.equals(JacobianPoint.ZERO) ? undefined : sum.toAffine();
    }
};
function sliceDER(s) {
    return Number.parseInt(s[0], 16) >= 8 ? '00' + s : s;
}
function parseDERInt(data) {
    if (data.length < 2 || data[0] !== 0x02) {
        throw new Error(`Invalid signature integer tag: ${bytesToHex$1(data)}`);
    }
    const len = data[1];
    const res = data.subarray(2, len + 2);
    if (!len || res.length !== len) {
        throw new Error(`Invalid signature integer: wrong length`);
    }
    if (res[0] === 0x00 && res[1] <= 0x7f) {
        throw new Error('Invalid signature integer: trailing length');
    }
    return { data: bytesToNumber(res), left: data.subarray(len + 2) };
}
function parseDERSignature(data) {
    if (data.length < 2 || data[0] != 0x30) {
        throw new Error(`Invalid signature tag: ${bytesToHex$1(data)}`);
    }
    if (data[1] !== data.length - 2) {
        throw new Error('Invalid signature: incorrect length');
    }
    const { data: r, left: sBytes } = parseDERInt(data.subarray(2));
    const { data: s, left: rBytesLeft } = parseDERInt(sBytes);
    if (rBytesLeft.length) {
        throw new Error(`Invalid signature: left bytes after parsing: ${bytesToHex$1(rBytesLeft)}`);
    }
    return { r, s };
}
class Signature {
    constructor(r, s) {
        this.r = r;
        this.s = s;
        this.assertValidity();
    }
    static fromCompact(hex) {
        const arr = hex instanceof Uint8Array;
        const name = 'Signature.fromCompact';
        if (typeof hex !== 'string' && !arr)
            throw new TypeError(`${name}: Expected string or Uint8Array`);
        const str = arr ? bytesToHex$1(hex) : hex;
        if (str.length !== 128)
            throw new Error(`${name}: Expected 64-byte hex`);
        return new Signature(hexToNumber(str.slice(0, 64)), hexToNumber(str.slice(64, 128)));
    }
    static fromDER(hex) {
        const arr = hex instanceof Uint8Array;
        if (typeof hex !== 'string' && !arr)
            throw new TypeError(`Signature.fromDER: Expected string or Uint8Array`);
        const { r, s } = parseDERSignature(arr ? hex : hexToBytes$1(hex));
        return new Signature(r, s);
    }
    static fromHex(hex) {
        return this.fromDER(hex);
    }
    assertValidity() {
        const { r, s } = this;
        if (!isWithinCurveOrder(r))
            throw new Error('Invalid Signature: r must be 0 < r < n');
        if (!isWithinCurveOrder(s))
            throw new Error('Invalid Signature: s must be 0 < s < n');
    }
    hasHighS() {
        const HALF = CURVE.n >> _1n;
        return this.s > HALF;
    }
    normalizeS() {
        return this.hasHighS() ? new Signature(this.r, mod(-this.s, CURVE.n)) : this;
    }
    toDERRawBytes() {
        return hexToBytes$1(this.toDERHex());
    }
    toDERHex() {
        const sHex = sliceDER(numberToHexUnpadded(this.s));
        const rHex = sliceDER(numberToHexUnpadded(this.r));
        const sHexL = sHex.length / 2;
        const rHexL = rHex.length / 2;
        const sLen = numberToHexUnpadded(sHexL);
        const rLen = numberToHexUnpadded(rHexL);
        const length = numberToHexUnpadded(rHexL + sHexL + 4);
        return `30${length}02${rLen}${rHex}02${sLen}${sHex}`;
    }
    toRawBytes() {
        return this.toDERRawBytes();
    }
    toHex() {
        return this.toDERHex();
    }
    toCompactRawBytes() {
        return hexToBytes$1(this.toCompactHex());
    }
    toCompactHex() {
        return numTo32bStr(this.r) + numTo32bStr(this.s);
    }
}
function concatBytes(...arrays) {
    if (!arrays.every((b) => b instanceof Uint8Array))
        throw new Error('Uint8Array list expected');
    if (arrays.length === 1)
        return arrays[0];
    const length = arrays.reduce((a, arr) => a + arr.length, 0);
    const result = new Uint8Array(length);
    for (let i = 0, pad = 0; i < arrays.length; i++) {
        const arr = arrays[i];
        result.set(arr, pad);
        pad += arr.length;
    }
    return result;
}
const hexes = Array.from({ length: 256 }, (_v, i) => i.toString(16).padStart(2, '0'));
function bytesToHex$1(uint8a) {
    if (!(uint8a instanceof Uint8Array))
        throw new Error('Expected Uint8Array');
    let hex = '';
    for (let i = 0; i < uint8a.length; i++) {
        hex += hexes[uint8a[i]];
    }
    return hex;
}
const POW_2_256 = BigInt('0x10000000000000000000000000000000000000000000000000000000000000000');
function numTo32bStr(num) {
    if (typeof num !== 'bigint')
        throw new Error('Expected bigint');
    if (!(_0n <= num && num < POW_2_256))
        throw new Error('Expected number 0 <= n < 2^256');
    return num.toString(16).padStart(64, '0');
}
function numTo32b(num) {
    const b = hexToBytes$1(numTo32bStr(num));
    if (b.length !== 32)
        throw new Error('Error: expected 32 bytes');
    return b;
}
function numberToHexUnpadded(num) {
    const hex = num.toString(16);
    return hex.length & 1 ? `0${hex}` : hex;
}
function hexToNumber(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToNumber: expected string, got ' + typeof hex);
    }
    return BigInt(`0x${hex}`);
}
function hexToBytes$1(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToBytes: expected string, got ' + typeof hex);
    }
    if (hex.length % 2)
        throw new Error('hexToBytes: received invalid unpadded hex' + hex.length);
    const array = new Uint8Array(hex.length / 2);
    for (let i = 0; i < array.length; i++) {
        const j = i * 2;
        const hexByte = hex.slice(j, j + 2);
        const byte = Number.parseInt(hexByte, 16);
        if (Number.isNaN(byte) || byte < 0)
            throw new Error('Invalid byte sequence');
        array[i] = byte;
    }
    return array;
}
function bytesToNumber(bytes) {
    return hexToNumber(bytesToHex$1(bytes));
}
function ensureBytes(hex) {
    return hex instanceof Uint8Array ? Uint8Array.from(hex) : hexToBytes$1(hex);
}
function normalizeScalar(num) {
    if (typeof num === 'number' && Number.isSafeInteger(num) && num > 0)
        return BigInt(num);
    if (typeof num === 'bigint' && isWithinCurveOrder(num))
        return num;
    throw new TypeError('Expected valid private scalar: 0 < scalar < curve.n');
}
function mod(a, b = CURVE.P) {
    const result = a % b;
    return result >= _0n ? result : b + result;
}
function pow2(x, power) {
    const { P } = CURVE;
    let res = x;
    while (power-- > _0n) {
        res *= res;
        res %= P;
    }
    return res;
}
function sqrtMod(x) {
    const { P } = CURVE;
    const _6n = BigInt(6);
    const _11n = BigInt(11);
    const _22n = BigInt(22);
    const _23n = BigInt(23);
    const _44n = BigInt(44);
    const _88n = BigInt(88);
    const b2 = (x * x * x) % P;
    const b3 = (b2 * b2 * x) % P;
    const b6 = (pow2(b3, _3n) * b3) % P;
    const b9 = (pow2(b6, _3n) * b3) % P;
    const b11 = (pow2(b9, _2n) * b2) % P;
    const b22 = (pow2(b11, _11n) * b11) % P;
    const b44 = (pow2(b22, _22n) * b22) % P;
    const b88 = (pow2(b44, _44n) * b44) % P;
    const b176 = (pow2(b88, _88n) * b88) % P;
    const b220 = (pow2(b176, _44n) * b44) % P;
    const b223 = (pow2(b220, _3n) * b3) % P;
    const t1 = (pow2(b223, _23n) * b22) % P;
    const t2 = (pow2(t1, _6n) * b2) % P;
    const rt = pow2(t2, _2n);
    const xc = (rt * rt) % P;
    if (xc !== x)
        throw new Error('Cannot find square root');
    return rt;
}
function invert(number, modulo = CURVE.P) {
    if (number === _0n || modulo <= _0n) {
        throw new Error(`invert: expected positive integers, got n=${number} mod=${modulo}`);
    }
    let a = mod(number, modulo);
    let b = modulo;
    let x = _0n, u = _1n;
    while (a !== _0n) {
        const q = b / a;
        const r = b % a;
        const m = x - u * q;
        b = a, a = r, x = u, u = m;
    }
    const gcd = b;
    if (gcd !== _1n)
        throw new Error('invert: does not exist');
    return mod(x, modulo);
}
function invertBatch(nums, p = CURVE.P) {
    const scratch = new Array(nums.length);
    const lastMultiplied = nums.reduce((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = acc;
        return mod(acc * num, p);
    }, _1n);
    const inverted = invert(lastMultiplied, p);
    nums.reduceRight((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = mod(acc * scratch[i], p);
        return mod(acc * num, p);
    }, inverted);
    return scratch;
}
function bits2int_2(bytes) {
    const delta = bytes.length * 8 - groupLen * 8;
    const num = bytesToNumber(bytes);
    return delta > 0 ? num >> BigInt(delta) : num;
}
function truncateHash(hash, truncateOnly = false) {
    const h = bits2int_2(hash);
    if (truncateOnly)
        return h;
    const { n } = CURVE;
    return h >= n ? h - n : h;
}
let _sha256Sync;
let _hmacSha256Sync;
function isWithinCurveOrder(num) {
    return _0n < num && num < CURVE.n;
}
function isValidFieldElement(num) {
    return _0n < num && num < CURVE.P;
}
function normalizePrivateKey(key) {
    let num;
    if (typeof key === 'bigint') {
        num = key;
    }
    else if (typeof key === 'number' && Number.isSafeInteger(key) && key > 0) {
        num = BigInt(key);
    }
    else if (typeof key === 'string') {
        if (key.length !== 2 * groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = hexToNumber(key);
    }
    else if (key instanceof Uint8Array) {
        if (key.length !== groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = bytesToNumber(key);
    }
    else {
        throw new TypeError('Expected valid private key');
    }
    if (!isWithinCurveOrder(num))
        throw new Error('Expected private key: 0 < key < n');
    return num;
}
function normalizePublicKey(publicKey) {
    if (publicKey instanceof Point$1) {
        publicKey.assertValidity();
        return publicKey;
    }
    else {
        return Point$1.fromHex(publicKey);
    }
}
function normalizeSignature(signature) {
    if (signature instanceof Signature) {
        signature.assertValidity();
        return signature;
    }
    try {
        return Signature.fromDER(signature);
    }
    catch (error) {
        return Signature.fromCompact(signature);
    }
}
function isProbPub(item) {
    const arr = item instanceof Uint8Array;
    const str = typeof item === 'string';
    const len = (arr || str) && item.length;
    if (arr)
        return len === compressedLen || len === uncompressedLen;
    if (str)
        return len === compressedLen * 2 || len === uncompressedLen * 2;
    if (item instanceof Point$1)
        return true;
    return false;
}
function getSharedSecret(privateA, publicB, isCompressed = false) {
    if (isProbPub(privateA))
        throw new TypeError('getSharedSecret: first arg must be private key');
    if (!isProbPub(publicB))
        throw new TypeError('getSharedSecret: second arg must be public key');
    const b = normalizePublicKey(publicB);
    b.assertValidity();
    return b.multiply(normalizePrivateKey(privateA)).toRawBytes(isCompressed);
}
Point$1.BASE._setWindowSize(8);
const TAGGED_HASH_PREFIXES = {};
const utils = {
    bytesToHex: bytesToHex$1,
    hexToBytes: hexToBytes$1,
    concatBytes,
    mod,
    invert,
    isValidPrivateKey(privateKey) {
        try {
            normalizePrivateKey(privateKey);
            return true;
        }
        catch (error) {
            return false;
        }
    },
    _bigintTo32Bytes: numTo32b,
    _normalizePrivateKey: normalizePrivateKey,
    hashToPrivateKey: (hash) => {
        hash = ensureBytes(hash);
        const minLen = groupLen + 8;
        if (hash.length < minLen || hash.length > 1024) {
            throw new Error(`Expected valid bytes of private key as per FIPS 186`);
        }
        const num = mod(bytesToNumber(hash), CURVE.n - _1n) + _1n;
        return numTo32b(num);
    },
    randomBytes: (bytesLength = 32) => {
        return crypto$2.getRandomValues(new Uint8Array(bytesLength));
    },
    randomPrivateKey: () => utils.hashToPrivateKey(utils.randomBytes(groupLen + 8)),
    precompute(windowSize = 8, point = Point$1.BASE) {
        const cached = point === Point$1.BASE ? point : new Point$1(point.x, point.y);
        cached._setWindowSize(windowSize);
        cached.multiply(_3n);
        return cached;
    },
    sha256: async (...messages) => {
        const buffer = await crypto$2.subtle.digest('SHA-256', concatBytes(...messages));
        return new Uint8Array(buffer);
    },
    hmacSha256: async (key, ...messages) => {
        const ckey = await crypto$2.subtle.importKey('raw', key, { name: 'HMAC', hash: { name: 'SHA-256' } }, false, ['sign']);
        const message = concatBytes(...messages);
        const buffer = await crypto$2.subtle.sign('HMAC', ckey, message);
        return new Uint8Array(buffer);
    },
    sha256Sync: undefined,
    hmacSha256Sync: undefined,
    taggedHash: async (tag, ...messages) => {
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = await utils.sha256(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return utils.sha256(tagP, ...messages);
    },
    taggedHashSync: (tag, ...messages) => {
        if (typeof _sha256Sync !== 'function')
            throw new ShaError('sha256Sync is undefined, you need to set it');
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = _sha256Sync(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return _sha256Sync(tagP, ...messages);
    },
    _JacobianPoint: JacobianPoint,
};
Object.defineProperties(utils, {
    sha256Sync: {
        configurable: false,
        get() {
            return _sha256Sync;
        },
        set(val) {
            if (!_sha256Sync)
                _sha256Sync = val;
        },
    },
    hmacSha256Sync: {
        configurable: false,
        get() {
            return _hmacSha256Sync;
        },
        set(val) {
            if (!_hmacSha256Sync)
                _hmacSha256Sync = val;
        },
    },
});

const ec$1 = new TextEncoder();
const ALPHABETS = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    },
    {
        name: 'base64',
        charset: 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
    },
    {
        name: 'base64url',
        charset: 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_'
    }
];
function getAlphabet(name) {
    for (const alpha of ALPHABETS) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1(data, charset, padding = false) {
    if (typeof data === 'string')
        data = ec$1.encode(data);
    const alphabet = getAlphabet(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1(encoded, charset) {
    const alphabet = getAlphabet(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        if (!(c > 0 || (b.length ^ i) > 0))
            b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return new Uint8Array(b);
}
const BaseX = {
    encode: encode$1,
    decode: decode$1
};

const CHARSET = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const encodings = {
    BECH32: 'bech32',
    BECH32M: 'bech32m'
};
function getEncodingConst(enc) {
    switch (enc) {
        case encodings.BECH32:
            return 1;
        case encodings.BECH32M:
            return 0x2bc830a3;
        default:
            throw new Error(`Unrecognized encoding: ${enc}`);
    }
}
function polymod(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) > 0) {
                chk ^= GENERATOR[i];
            }
        }
    }
    return chk;
}
function hrpExpand(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function convertBits(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            return [];
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        return [];
    }
    return ret;
}
function verifyChecksum(hrp, data, enc) {
    const combined = hrpExpand(hrp).concat(data);
    return polymod(combined) === getEncodingConst(enc);
}
function createChecksum(hrp, data, enc) {
    const values = hrpExpand(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod(values) ^ getEncodingConst(enc);
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function b32encode(hrp, data, enc) {
    const combined = data.concat(createChecksum(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET.charAt(combined[p]);
    }
    return ret;
}
function b32decode(bechstr, version) {
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    if (!checkBounds(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET.indexOf(bechstr.charAt(p));
        if (d === -1) {
            return { hrp: null, data: [0xFF] };
        }
        data.push(d);
    }
    return (verifyChecksum(hrp, data, enc))
        ? { hrp, data: data.slice(0, data.length - 6) }
        : { hrp: null, data: [0xFF] };
}
function checkBounds(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    return !(hasLower && hasUpper);
}
function checkSeparatorPos(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function encode(data, hrp = 'bch', version = 0) {
    const dat = [version, ...convertBits([...data], 8, 5)];
    const enc = (version > 0) ? 'bech32m' : 'bech32';
    const str = b32encode(hrp, dat, enc);
    decode(str, version);
    return str;
}
function decode(str, version = 0) {
    const hrp = str.split('1', 1)[0];
    const { hrp: hrpgot, data } = b32decode(str, version);
    const decoded = convertBits(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        case (data[0] === 0 && length !== 20 && length !== 32):
            throw new Error('Decoded string does not match version 0 spec.');
        case (data[0] === 0 && version !== 0):
            throw new Error('Decoded version bit does not match.');
        case (data[0] !== 0 && version !== 1):
            throw new Error('Decoded version bit does not match.');
        default:
            return Uint8Array.from(decoded);
    }
}
const Bech32 = {
    encode,
    decode
};

const ec = new TextEncoder();
const dc = new TextDecoder();
function strToBytes(str) {
    return ec.encode(str);
}
function hexToBytes(str) {
    const bytes = [];
    let i, idx = 0;
    if (str.length % 2 > 0) {
        throw new Error(`Invalid hex string length: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return Uint8Array.from(bytes);
}
function numToBytes(num) {
    if (num === 0)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return Uint8Array.from(bytes);
}
function bigToBytes(big) {
    if (big === 0n)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return Uint8Array.from(bytes);
}
function bytesToStr(bytes) {
    return dc.decode(bytes);
}
function bytesToHex(bytes) {
    const hex = [];
    let i;
    for (i = 0; i < bytes.length; i++) {
        hex.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return hex.join('');
}
function bytesToNum(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer(value) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    const type = typeof value;
    switch (type) {
        case 'bigint':
            return bigToBytes(value);
        case 'boolean':
            return Uint8Array.of(value);
        case 'number':
            return numToBytes(value);
        case 'string':
            return (value.match(/^(02|03)*[0-9a-fA-F]{64}$/) !== null)
                ? hexToBytes(value)
                : ec.encode(value);
        default:
            throw TypeError('Unsupported format:' + type);
    }
}

const digestLength = 32;
const blockSize = 64;
const K = new Uint32Array([
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b,
    0x59f111f1, 0x923f82a4, 0xab1c5ed5, 0xd807aa98, 0x12835b01,
    0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7,
    0xc19bf174, 0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
    0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da, 0x983e5152,
    0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147,
    0x06ca6351, 0x14292967, 0x27b70a85, 0x2e1b2138, 0x4d2c6dfc,
    0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819,
    0xd6990624, 0xf40e3585, 0x106aa070, 0x19a4c116, 0x1e376c08,
    0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f,
    0x682e6ff3, 0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
    0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
]);
function hashBlocks(w, v, p, pos, len) {
    let a, b, c, d, e, f, g, h, u, i, j, t1, t2;
    while (len >= 64) {
        a = v[0];
        b = v[1];
        c = v[2];
        d = v[3];
        e = v[4];
        f = v[5];
        g = v[6];
        h = v[7];
        for (i = 0; i < 16; i++) {
            j = pos + i * 4;
            w[i] = (((p[j] & 0xff) << 24) | ((p[j + 1] & 0xff) << 16) |
                ((p[j + 2] & 0xff) << 8) | (p[j + 3] & 0xff));
        }
        for (i = 16; i < 64; i++) {
            u = w[i - 2];
            t1 = (u >>> 17 | u << (32 - 17)) ^ (u >>> 19 | u << (32 - 19)) ^ (u >>> 10);
            u = w[i - 15];
            t2 = (u >>> 7 | u << (32 - 7)) ^ (u >>> 18 | u << (32 - 18)) ^ (u >>> 3);
            w[i] = (t1 + w[i - 7] | 0) + (t2 + w[i - 16] | 0);
        }
        for (i = 0; i < 64; i++) {
            t1 = (((((e >>> 6 | e << (32 - 6)) ^ (e >>> 11 | e << (32 - 11)) ^
                (e >>> 25 | e << (32 - 25))) + ((e & f) ^ (~e & g))) | 0) +
                ((h + ((K[i] + w[i]) | 0)) | 0)) | 0;
            t2 = (((a >>> 2 | a << (32 - 2)) ^ (a >>> 13 | a << (32 - 13)) ^
                (a >>> 22 | a << (32 - 22))) + ((a & b) ^ (a & c) ^ (b & c))) | 0;
            h = g;
            g = f;
            f = e;
            e = (d + t1) | 0;
            d = c;
            c = b;
            b = a;
            a = (t1 + t2) | 0;
        }
        v[0] += a;
        v[1] += b;
        v[2] += c;
        v[3] += d;
        v[4] += e;
        v[5] += f;
        v[6] += g;
        v[7] += h;
        pos += 64;
        len -= 64;
    }
    return pos;
}
let Hash$1 = class Hash {
    constructor() {
        this.digestLength = digestLength;
        this.blockSize = blockSize;
        this.state = new Int32Array(8);
        this.temp = new Int32Array(64);
        this.buffer = new Uint8Array(128);
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        this.reset();
    }
    reset() {
        this.state[0] = 0x6a09e667;
        this.state[1] = 0xbb67ae85;
        this.state[2] = 0x3c6ef372;
        this.state[3] = 0xa54ff53a;
        this.state[4] = 0x510e527f;
        this.state[5] = 0x9b05688c;
        this.state[6] = 0x1f83d9ab;
        this.state[7] = 0x5be0cd19;
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        return this;
    }
    clean() {
        for (let i = 0; i < this.buffer.length; i++) {
            this.buffer[i] = 0;
        }
        for (let i = 0; i < this.temp.length; i++) {
            this.temp[i] = 0;
        }
        this.reset();
    }
    update(data, dataLength = data.length) {
        if (this.finished) {
            throw new Error("SHA256: can't update because hash was finished.");
        }
        let dataPos = 0;
        this.bytesHashed += dataLength;
        if (this.bufferLength > 0) {
            while (this.bufferLength < 64 && dataLength > 0) {
                this.buffer[this.bufferLength++] = data[dataPos++];
                dataLength--;
            }
            if (this.bufferLength === 64) {
                hashBlocks(this.temp, this.state, this.buffer, 0, 64);
                this.bufferLength = 0;
            }
        }
        if (dataLength >= 64) {
            dataPos = hashBlocks(this.temp, this.state, data, dataPos, dataLength);
            dataLength %= 64;
        }
        while (dataLength > 0) {
            this.buffer[this.bufferLength++] = data[dataPos++];
            dataLength--;
        }
        return this;
    }
    finish(out) {
        if (!this.finished) {
            const bytesHashed = this.bytesHashed;
            const left = this.bufferLength;
            const bitLenHi = (bytesHashed / 0x20000000) | 0;
            const bitLenLo = bytesHashed << 3;
            const padLength = (bytesHashed % 64 < 56) ? 64 : 128;
            this.buffer[left] = 0x80;
            for (let i = left + 1; i < padLength - 8; i++) {
                this.buffer[i] = 0;
            }
            this.buffer[padLength - 8] = (bitLenHi >>> 24) & 0xff;
            this.buffer[padLength - 7] = (bitLenHi >>> 16) & 0xff;
            this.buffer[padLength - 6] = (bitLenHi >>> 8) & 0xff;
            this.buffer[padLength - 5] = (bitLenHi >>> 0) & 0xff;
            this.buffer[padLength - 4] = (bitLenLo >>> 24) & 0xff;
            this.buffer[padLength - 3] = (bitLenLo >>> 16) & 0xff;
            this.buffer[padLength - 2] = (bitLenLo >>> 8) & 0xff;
            this.buffer[padLength - 1] = (bitLenLo >>> 0) & 0xff;
            hashBlocks(this.temp, this.state, this.buffer, 0, padLength);
            this.finished = true;
        }
        for (let i = 0; i < 8; i++) {
            out[i * 4 + 0] = (this.state[i] >>> 24) & 0xff;
            out[i * 4 + 1] = (this.state[i] >>> 16) & 0xff;
            out[i * 4 + 2] = (this.state[i] >>> 8) & 0xff;
            out[i * 4 + 3] = (this.state[i] >>> 0) & 0xff;
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
    _saveState(out) {
        for (let i = 0; i < this.state.length; i++) {
            out[i] = this.state[i];
        }
    }
    _restoreState(from, bytesHashed) {
        for (let i = 0; i < this.state.length; i++) {
            this.state[i] = from[i];
        }
        this.bytesHashed = bytesHashed;
        this.finished = false;
        this.bufferLength = 0;
    }
};
class HMAC {
    constructor(key) {
        this.inner = new Hash$1();
        this.outer = new Hash$1();
        this.blockSize = this.inner.blockSize;
        this.digestLength = this.inner.digestLength;
        const pad = new Uint8Array(this.blockSize);
        if (key.length > this.blockSize) {
            (new Hash$1()).update(key).finish(pad).clean();
        }
        else {
            for (let i = 0; i < key.length; i++) {
                pad[i] = key[i];
            }
        }
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36;
        }
        this.inner.update(pad);
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36 ^ 0x5c;
        }
        this.outer.update(pad);
        this.istate = new Uint32Array(8);
        this.ostate = new Uint32Array(8);
        this.inner._saveState(this.istate);
        this.outer._saveState(this.ostate);
        for (let i = 0; i < pad.length; i++) {
            pad[i] = 0;
        }
    }
    reset() {
        this.inner._restoreState(this.istate, this.inner.blockSize);
        this.outer._restoreState(this.ostate, this.outer.blockSize);
        return this;
    }
    clean() {
        for (let i = 0; i < this.istate.length; i++) {
            this.ostate[i] = this.istate[i] = 0;
        }
        this.inner.clean();
        this.outer.clean();
    }
    update(data) {
        this.inner.update(data);
        return this;
    }
    finish(out) {
        if (this.outer.finished) {
            this.outer.finish(out);
        }
        else {
            this.inner.finish(out);
            this.outer.update(out, this.digestLength).finish(out);
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
}
function sha256$1(data) {
    const h = (new Hash$1()).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}
function hash256$1(data) {
    return sha256$1(sha256$1(data));
}
function hmac256$1(key, data) {
    const h = (new HMAC(key)).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}

function addChecksum(data) {
    const sum = hash256$1(data);
    return Uint8Array.of(...data, ...sum.slice(0, 4));
}
function checkTheSum(data) {
    const ret = data.slice(0, -4);
    const chk = data.slice(-4);
    const sum = hash256$1(ret).slice(0, 4);
    if (sum.toString() !== chk.toString()) {
        throw new Error('Invalid checksum!');
    }
    return ret;
}
class Buff extends Uint8Array {
    static { this.num = (number, size, endian = 'le') => {
        const b = new Buff(numToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.big = (number, size, endian = 'le') => {
        const b = new Buff(bigToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static b58check(data) {
        const decoded = BaseX.decode(data, 'base58');
        return new Buff(checkTheSum(decoded));
    }
    static { this.any = (data, size) => new Buff(buffer(data), size); }
    static { this.raw = (data, size) => new Buff(data, size); }
    static { this.str = (data, size) => new Buff(strToBytes(data), size); }
    static { this.hex = (data, size) => new Buff(hexToBytes(data), size); }
    static { this.json = (data) => new Buff(strToBytes(JSON.stringify(data))); }
    static { this.base64 = (data) => new Buff(BaseX.decode(data, 'base64')); }
    static { this.b64url = (data) => new Buff(BaseX.decode(data, 'base64url')); }
    static { this.bech32 = (data, ver = 0) => new Buff(Bech32.decode(data, ver)); }
    static { this.bech32m = (data) => new Buff(Bech32.decode(data, 1)); }
    constructor(data, size) {
        if (size !== undefined) {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        return this;
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get arr() {
        return this.toArr();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get base64() {
        return this.toBase64();
    }
    get b64url() {
        return this.toB64url();
    }
    get digest() {
        return this.toDigest();
    }
    get id() {
        return this.toDigest().hex;
    }
    toNum(endian = 'le') {
        return (endian === 'le')
            ? bytesToNum(this.reverse())
            : bytesToNum(this);
    }
    toBig(endian = 'le') {
        return (endian === 'le')
            ? bytesToBig(this.reverse())
            : bytesToBig(this);
    }
    toDigest() {
        return new Buff(sha256$1(this));
    }
    toHmac(key) {
        if (typeof key === 'string') {
            key = strToBytes(key);
        }
        return new Buff(hmac256$1(key, this));
    }
    tob58check() {
        return BaseX.encode(addChecksum(this), 'base58');
    }
    toArr() { return Array.from(this); }
    toStr() { return bytesToStr(this); }
    toHex() { return bytesToHex(this); }
    toJson() { return JSON.parse(bytesToStr(this)); }
    toBytes() { return new Uint8Array(this); }
    toB64url() { return BaseX.encode(this, 'base64url'); }
    toBase64(padding) { return BaseX.encode(this, 'base64', padding); }
    toBech32(hrp, ver = 0) { return Bech32.encode(this, hrp, ver); }
    toBech32m(hrp) { return Bech32.encode(this, hrp, 1); }
    prepend(data) {
        return Buff.of(...data, ...this);
    }
    append(data) {
        return Buff.of(...this, ...data);
    }
    slice(start, end) {
        return new Buff(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prefixSize(endian) {
        return Buff.of(...Buff.readSize(this.length, endian), ...this);
    }
    static from(data) {
        return new Buff(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff(Uint8Array.of(...args));
    }
    static join(arr) {
        let i, idx = 0;
        const totalSize = arr.reduce((prev, curr) => prev + curr.length, 0);
        const totalBytes = new Uint8Array(totalSize);
        for (const bytes of arr) {
            for (i = 0; i < bytes.length; idx++, i++) {
                totalBytes[idx] = bytes[i];
            }
        }
        return new Buff(totalBytes, totalSize);
    }
    static readSize(num, endian) {
        if (num < 0xFD) {
            return Buff.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff.of(0xFD, ...Buff.num(num, 2, endian));
        }
        else if (num < 0x100000000) {
            return Buff.of(0xFE, ...Buff.num(num, 4, endian));
        }
        else if (num < 0x10000000000000000) {
            return Buff.of(0xFF, ...Buff.num(num, 8, endian));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static random(size = 32) {
        return new Buff(crypto.getRandomValues(new Uint8Array(size)));
    }
    static { this.encode = strToBytes; }
    static { this.decode = bytesToStr; }
    static normalize(data, size) {
        if (data instanceof Uint8Array)
            return data;
        if (typeof data === 'string')
            return Buff.hex(data, size).raw;
        if (typeof data === 'number')
            return Buff.num(data, size).raw;
        if (typeof data === 'bigint')
            return Buff.big(data, size).raw;
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static serialize(data) {
        if (typeof data === 'string') {
            return Buff.str(data).raw;
        }
        if (typeof data === 'object') {
            if (data instanceof Uint8Array) {
                return data;
            }
            try {
                return Buff.json(data).raw;
            }
            catch {
                throw TypeError('Object is not serializable.');
            }
        }
        throw TypeError(`Unrecognized format: ${typeof data}`);
    }
    static revitalize(data) {
        if (data instanceof Uint8Array) {
            data = bytesToStr(data);
        }
        if (typeof data === 'string') {
            try {
                return JSON.parse(data);
            }
            catch {
                return data;
            }
        }
        return data;
    }
}

const crypto$1 = getCryptoLib();
function getCryptoLib() {
    if (typeof globalThis !== 'undefined') {
        return globalThis.crypto;
    }
    if (typeof window !== 'undefined') {
        return window.crypto;
    }
    throw new Error('Must be using https in browser, or node/deno environment that supports globalthis.crypto object!');
}
async function ecdh(secretKey, sharedKey) {
    const secret = Buff.normalize(secretKey);
    const shared = Buff.normalize(sharedKey);
    return getSharedSecret(secret, shared, true);
}
async function importCryptoKey(key) {
    const options = { name: 'AES-CBC' };
    const usage = ['encrypt', 'decrypt'];
    return crypto$1.subtle.importKey('raw', Buff.normalize(key), options, true, usage);
}
async function exportCryptoKey(key) {
    return crypto$1.subtle.exportKey('raw', key)
        .then(buff => new Uint8Array(buff));
}
async function importHmacKey(key, fmt = 'SHA-256') {
    const config = { name: 'HMAC', hash: fmt };
    return crypto$1.subtle.importKey('raw', Buff.normalize(key), config, false, ['sign', 'verify']);
}
async function generateCryptoKey() {
    return importCryptoKey(Buff.random(32));
}
async function normalizeCryptoKey(key) {
    if (key instanceof CryptoKey)
        return key;
    return KeyUtil.import(key);
}
async function getSharedCryptoKey(secretKey, sharedKey) {
    const bytes = await ecdh(secretKey, sharedKey);
    return importCryptoKey(bytes.slice(1, 33));
}
const KeyUtil = {
    ecdh,
    import: importCryptoKey,
    export: exportCryptoKey,
    hmac: importHmacKey,
    shared: getSharedCryptoKey,
    generate: generateCryptoKey,
    normalize: normalizeCryptoKey
};

class Field extends Uint8Array {
    static { this.N = CURVE.n; }
    static mod(n, m = Field.N) {
        return utils.mod(n, m);
    }
    static normalize(num) {
        num = normalizeField(num);
        num = Field.mod(num);
        Field.validate(num);
        return Buff.big(num, 32);
    }
    static validate(num) {
        return utils.isValidPrivateKey(num);
    }
    constructor(x) {
        super(Field.normalize(x));
    }
    get buff() {
        return new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get num() {
        return this.buff.big;
    }
    get big() {
        return this.buff.big;
    }
    get hex() {
        return this.buff.hex;
    }
    get point() {
        return this.generate();
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    get negated() {
        return (this.hasOddY)
            ? this.negate()
            : this;
    }
    gt(num) {
        const x = new Field(num);
        return x.num > this.num;
    }
    lt(num) {
        const x = new Field(num);
        return x.num < this.num;
    }
    eq(num) {
        const x = new Field(num);
        return x.num === this.num;
    }
    ne(num) {
        const x = new Field(num);
        return x.num !== this.num;
    }
    add(num) {
        const x = new Field(num);
        return new Field(this.num + x.num);
    }
    sub(num) {
        const x = new Field(num);
        return new Field(this.num - x.num);
    }
    mul(num) {
        const x = new Field(num);
        return new Field(this.num * x.num);
    }
    pow(num, n = Field.N - 1n) {
        const x = new Field(num);
        const e = Field.mod(x.num, n);
        return new Field(this.num ** e);
    }
    div(num) {
        const x = new Field(num);
        const d = this.pow(x.num, Field.N - 2n);
        return new Field(this.num * d.num);
    }
    negate() {
        return new Field(Field.N - this.num);
    }
    generate() {
        return Point.import(Point$1.BASE.multiply(this.big));
    }
}
class Point {
    static { this.N = CURVE.n; }
    static validate(x) {
        try {
            x = new Point(x);
            return true;
        }
        catch {
            return false;
        }
    }
    static normalize(x) {
        x = normalizePoint(x);
        return Point$1.fromHex(x);
    }
    static generate(num) {
        return new Field(num).generate();
    }
    static import(point) {
        return new Point(point.x, point.y);
    }
    constructor(x, y) {
        this.__p = (typeof x === 'bigint' &&
            typeof y === 'bigint')
            ? new Point$1(x, y)
            : Point.normalize(x);
        this.p.assertValidity();
    }
    get p() {
        return this.__p;
    }
    get x() {
        return this.p.x;
    }
    get y() {
        return this.p.y;
    }
    get buff() {
        const p = this.p.toRawBytes(true);
        if (p.length < 33) {
            const prefix = this.p.hasEvenY() ? 0x02 : 0x03;
            return Buff.of(prefix, ...p);
        }
        else {
            return Buff.raw(p);
        }
    }
    get buffX() {
        return new Buff(getXonly(this.buff));
    }
    get raw() {
        return this.buff.raw;
    }
    get rawX() {
        return this.buffX.raw;
    }
    get rawY() {
        return Buff.big(this.y).raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get hexX() {
        return this.buffX.hex;
    }
    get hasEvenY() {
        return this.p.hasEvenY();
    }
    get hasOddY() {
        return !this.p.hasEvenY();
    }
    eq(x) {
        return (x instanceof Point)
            ? this.p.equals(new Point$1(x.x, x.y))
            : (x instanceof Uint8Array)
                ? this.x.toString() === x.toString()
                : (typeof x === 'number')
                    ? BigInt(x) === this.x
                    : x === this.x;
    }
    add(x) {
        return (x instanceof Point)
            ? Point.import(this.p.add(x.p))
            : Point.import(this.p.add(Point.generate(x).p));
    }
    sub(x) {
        return (x instanceof Point)
            ? Point.import(this.p.subtract(x.p))
            : Point.import(this.p.subtract(Point.generate(x).p));
    }
    mul(x) {
        return (x instanceof Point)
            ? Point.import(this.p.multiply(x.x))
            : Point.import(this.p.multiply(normalizeField(x)));
    }
    negate() {
        return Point.import(this.p.negate());
    }
}
function getXonly(x) {
    return (x.length > 32) ? x.slice(1, 33) : x;
}
function normalizeField(value) {
    if (value instanceof Field) {
        return value.num;
    }
    if (value instanceof Point) {
        return value.x;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).big;
    }
    if (typeof value === 'string') {
        return Buff.hex(value).big;
    }
    if (typeof value === 'number') {
        return Buff.num(value).big;
    }
    if (typeof value === 'bigint') {
        return BigInt(value);
    }
    throw TypeError('Invalid input type:' + typeof value);
}
function normalizePoint(value) {
    if (value instanceof Field) {
        return value.hex;
    }
    if (value instanceof Point) {
        return value.hex;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).hex;
    }
    if (typeof value === 'string') {
        return value;
    }
    if (typeof value === 'number') {
        return Buff.num(value).hex;
    }
    if (typeof value === 'bigint') {
        return Buff.big(value).hex;
    }
    throw TypeError('Invalid input type:' + typeof value);
}

function bytesToBigInt(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML[i])];
        kl = KL[Number(rnd)];
        mr = x[Number(MR[i])];
        kr = KR[Number(rnd)];
        al = rol(al + fi(bl, cl, dl, rnd) + ml + kl, RL[i]) + el;
        elt = el;
        el = dl;
        dl = rol(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol(ar + fi(br, cr, dr, 4n - rnd) + mr + kr, RR[i]) + er;
        ert = er;
        er = dr;
        dr = rol(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes(num, 4));
    }
    return Uint8Array.from(ret);
}

async function digest(bytes, format = 'SHA-256', rounds = 1, hook = (buff) => buff) {
    let i;
    let buffer = (bytes instanceof ArrayBuffer)
        ? bytes
        : Buff.normalize(bytes).buffer;
    for (i = 0; i < rounds; i++) {
        buffer = await crypto$1.subtle.digest(format, buffer);
        hook(buffer);
    }
    return new Uint8Array(buffer);
}
async function sha256(bytes) {
    return digest(bytes, 'SHA-256');
}
async function sha512(bytes) {
    return digest(bytes, 'SHA-512');
}
function ripe160(bytes) {
    bytes = Buff.normalize(bytes);
    return ripemd160(bytes);
}
async function hash256(bytes) {
    return digest(bytes, 'SHA-256', 2);
}
async function hash160(bytes) {
    return ripe160(await sha256(bytes));
}
async function hmac256(key, data) {
    const keyfile = await KeyUtil.hmac(key, 'SHA-256');
    return crypto$1.subtle
        .sign('HMAC', keyfile, Buff.normalize(data))
        .then((buffer) => new Uint8Array(buffer));
}
async function hmac512(key, data) {
    const keyfile = await KeyUtil.hmac(key, 'SHA-512');
    return crypto$1.subtle
        .sign('HMAC', keyfile, Buff.normalize(data))
        .then((buffer) => new Uint8Array(buffer));
}
async function data(data) {
    const buff = Buff.serialize(data);
    return sha256(buff).then(buff => new Uint8Array(buff));
}
const Hash = {
    data,
    digest,
    ripe160,
    sha256,
    sha512,
    hash160,
    hash256,
    hmac256,
    hmac512,
    sync: {
        sha256: sha256$1,
        hash256: hash256$1,
        hmac256: hmac256$1
    }
};

const VALID_HASH_TYPES$1 = [0x01, 0x02, 0x03, 0x81, 0x82, 0x83];
async function segwitHash(txdata, idx, value, script, sigflag) {
    if (!VALID_HASH_TYPES$1.includes(sigflag)) {
        throw new Error('Invalid hash type: ' + String(sigflag));
    }
    const { version, input, output, locktime } = txdata;
    const { txid, vout, sequence } = input[idx];
    const isAnypay = sigflag > 0x80;
    const stack = [encodeVersion(version)];
    stack.push(await hashPrevouts(input, isAnypay), await hashSequence$1(input, sigflag), encodeTxid(txid), encodePrevOut(vout), encodeScript(script, true), encodeValue(value), encodeSequence(sequence), await hashOutputs$1(output, idx, sigflag), encodeLocktime(locktime), Buff$1.num(sigflag, 4).reverse());
    return Hash.hash256(Buff$1.join(stack));
}
async function hashPrevouts(vin, isAnypay) {
    if (isAnypay === true) {
        return Buff$1.num(0, 32);
    }
    const stack = [];
    for (const { txid, vout } of vin) {
        stack.push(encodeTxid(txid));
        stack.push(encodePrevOut(vout));
    }
    return Hash.hash256(Buff$1.join(stack));
}
async function hashSequence$1(vin, sigflag) {
    if (sigflag !== 0x01) {
        return Buff$1.num(0, 32);
    }
    const stack = [];
    for (const { sequence } of vin) {
        stack.push(encodeSequence(sequence));
    }
    return Hash.hash256(Buff$1.join(stack));
}
async function hashOutputs$1(vout, idx, sigflag) {
    const stack = [];
    if (sigflag === 0x01) {
        for (const { value, scriptPubKey } of vout) {
            stack.push(encodeValue(value));
            stack.push(encodeScript(scriptPubKey));
        }
        return Hash.hash256(Buff$1.join(stack));
    }
    if (sigflag === 0x03 && idx < vout.length) {
        const { value, scriptPubKey } = vout[idx];
        stack.push(encodeValue(value));
        stack.push(encodeScript(scriptPubKey));
        return Hash.hash256(Buff$1.join(stack));
    }
    return Buff$1.num(0, 32);
}

const DEFAULT_VERSION$1 = 0xc0;
function getTapTag(tag) {
    const htag = Buff$1.str(tag).digest;
    return Buff$1.join([htag, htag]);
}
function getTapLeaf(data, version = DEFAULT_VERSION$1) {
    return Buff$1.join([
        getTapTag('TapLeaf'),
        getVersion(version),
        Buff$1.normalize(data)
    ]).digest.hex;
}
function getTapBranch(leafA, leafB) {
    if (leafB < leafA) {
        [leafA, leafB] = [leafB, leafA];
    }
    return Buff$1.join([
        getTapTag('TapBranch'),
        Buff$1.hex(leafA).raw,
        Buff$1.hex(leafB).raw
    ]).digest.hex;
}
function getTapRoot(leaves) {
    return Buff$1.hex(merkleize(leaves)[0]);
}
function decodeTapAddress(address) {
    return Buff$1.bech32(address, 1);
}
function encodeTapAddress(tapkey, prefix = 'bc') {
    tapkey = Buff$1.normalize(tapkey);
    if (tapkey.length > 32) {
        tapkey = tapkey.slice(1, 33);
    }
    return Buff$1.raw(tapkey).toBech32(prefix, 1);
}
function merkleize(taptree, target = null, path = []) {
    const leaves = [];
    const tree = [];
    for (let i = 0; i < taptree.length; i++) {
        const leaf = taptree[i];
        if (Array.isArray(leaf)) {
            const [r, t, p] = merkleize(leaf, target);
            target = t;
            leaves.push(r);
            for (const e of p) {
                path.push(e);
            }
        }
        else {
            leaves.push(leaf);
        }
    }
    if (leaves.length === 1) {
        return [leaves[0], target, path];
    }
    leaves.sort();
    if (leaves.length % 2 !== 0) {
        leaves.push(leaves[leaves.length - 1]);
    }
    for (let i = 0; i < leaves.length - 1; i += 2) {
        const branch = getTapBranch(leaves[i], leaves[i + 1]);
        tree.push(branch);
        if (typeof target === 'string') {
            if (target === leaves[i]) {
                path.push(leaves[i + 1]);
                target = branch;
            }
            else if (target === leaves[i + 1]) {
                path.push(leaves[i]);
                target = branch;
            }
        }
    }
    return merkleize(tree, target, path);
}
function getVersion(version = 0xc0) {
    return version & 0xfe;
}

function tweakPrvkey(prvkey, tweak) {
    let sec = new Field(prvkey);
    if (sec.point.hasOddY) {
        sec = sec.negate();
    }
    return sec.add(tweak).raw;
}
function tweakPubkey(pubkey, tweak) {
    const P = new Point(pubkey);
    const Q = P.add(tweak);
    return Q.raw;
}
function getTapTweak(pubkey, tweak) {
    return Buff$1.join([
        getTapTag('TapTweak'),
        Buff$1.normalize(pubkey),
        Buff$1.normalize(tweak)
    ]).digest;
}
function getTapKey(intkey, leaves = [], isPrivate = false) {
    const k = Buff$1.normalize(intkey);
    const r = (leaves.length > 0)
        ? getTapRoot(leaves)
        : new Uint8Array();
    const P = (isPrivate)
        ? new Field(k).point.rawX
        : k;
    const t = getTapTweak(P, r);
    if (isPrivate) {
        return [Buff$1.raw(tweakPrvkey(k, t)).hex, 0];
    }
    else {
        const p = Buff$1.raw(tweakPubkey(k, t));
        return [p.slice(1).hex, p.slice(0, 1).num];
    }
}
function getTapPubkey(pubkey, leaves = []) {
    return getTapKey(pubkey, leaves);
}
function getTapSeckey(seckey, leaves = []) {
    return getTapKey(seckey, leaves, true)[0];
}

const DEFAULT_VERSION = 0xc0;
function getTapPath(pubkey, target, taptree = [target], version = DEFAULT_VERSION, parity = 0) {
    const [pub, par] = parsePubkey(pubkey);
    parity = (par !== undefined) ? par : parity;
    const ctrl = Buff$1.num(version + getParityBit(parity));
    const block = [ctrl, pub];
    const [root, _t, path] = merkleize(taptree, target);
    if (taptree.length > 1) {
        path.forEach(e => block.push(Buff$1.hex(e)));
    }
    const cblock = Buff$1.join(block);
    const tweak = getTapTweak(pub, Buff$1.hex(root));
    const tapkey = tweakPubkey(pub, tweak).slice(1);
    if (!checkTapPath(tapkey, cblock, target)) {
        if (parity === 0) {
            return getTapPath(pubkey, target, taptree, version, 1);
        }
        throw new Error('Path checking failed! Unable to generate path.');
    }
    return cblock.hex;
}
function checkTapPath(tapkey, cblock, target) {
    const buffer = new Stream(Buff$1.normalize(cblock));
    const [_v, y] = decodeCByte(buffer.read(1).num);
    const intkey = buffer.read(32);
    const pubkey = Buff$1.join([y, Buff$1.normalize(tapkey)]);
    const path = [];
    let branch = target;
    while (buffer.size >= 32) {
        path.push(buffer.read(32).hex);
    }
    if (buffer.size !== 0) {
        throw new Error('Invalid control block size!');
    }
    for (const p of path) {
        branch = getTapBranch(branch, p);
    }
    const t = getTapTweak(intkey, Buff$1.hex(branch));
    const k = tweakPubkey(intkey, t);
    return (Buff$1.raw(k).hex === Buff$1.raw(pubkey).hex);
}
function getParityBit(parity = 0x02) {
    if (parity === 0 || parity === 1)
        return parity;
    if (parity === 0x02 || parity === '02')
        return 0;
    if (parity === 0x03 || parity === '03')
        return 1;
    throw new Error('Invalid parity bit:' + String(parity));
}
function decodeCByte(byte) {
    return (byte % 2 === 0) ? [byte, 0x02] : [byte - 1, 0x03];
}
function parsePubkey(pubkey) {
    let parity;
    pubkey = Buff$1.normalize(pubkey);
    if (pubkey.length > 32) {
        parity = pubkey[0];
        pubkey = pubkey.slice(1, 33);
    }
    return [pubkey, parity];
}

function safeThrow(errorMsg, shouldThrow) {
    if (shouldThrow) {
        throw new Error(errorMsg);
    }
    else {
        return false;
    }
}
function hashTag(tag, ...data) {
    const htag = Buff$1.str(tag).digest.raw;
    const buff = data.map(e => Buff$1.normalize(e));
    return Buff$1.join([htag, htag, Buff$1.join(buff)]).digest;
}

const FIELD_SIZE = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2fn;
const CURVE_ORDER = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141n;
function sign(secret, message, rand = Buff$1.random(32)) {
    const m = Buff$1.normalize(message);
    const dp = new Field(secret);
    const P = dp.point;
    const d = (P.hasEvenY) ? dp.big : dp.negated.big;
    const a = hashTag('BIP0340/aux', Buff$1.normalize(rand));
    const t = d ^ a.big;
    const n = hashTag('BIP0340/nonce', t, P.rawX, m);
    const kp = new Field(n);
    const R = kp.point;
    const k = (R.hasEvenY) ? kp.big : kp.negated.big;
    const e = new Field(hashTag('BIP0340/challenge', R.rawX, P.rawX, m));
    const s = new Field(k + (e.big * d));
    return Buff$1.join([R.rawX, s.raw]);
}
function verify(signature, message, pubkey, shouldThrow = false) {
    const P = new Point(pubkey);
    const m = Buff$1.normalize(message);
    const stream = Buff$1.bytes(signature).stream;
    if (stream.size < 64) {
        safeThrow('Signature length is too small: ' + String(stream.size), shouldThrow);
    }
    const r = stream.read(32);
    if (r.big > FIELD_SIZE) {
        safeThrow('Signature r value greater than field size!', shouldThrow);
    }
    const s = stream.read(32);
    if (s.big > CURVE_ORDER) {
        safeThrow('Signature s value greater than curve order!', shouldThrow);
    }
    const e = new Field(hashTag('BIP0340/challenge', r.raw, P.rawX, m));
    const sG = new Field(s).point;
    const eP = P.mul(e.big);
    const R = sG.sub(eP);
    if (R.hasOddY) {
        safeThrow('Signature R value has odd Y coordinate!', shouldThrow);
    }
    if (R.x === 0n) {
        safeThrow('Signature R value is infinite!', shouldThrow);
    }
    return R.x === r.big;
}

const VALID_HASH_TYPES = [0x00, 0x01, 0x02, 0x03, 0x81, 0x82, 0x83];
function signTx(prvkey, txdata, index, config = {}) {
    const { sigflag = 0x00 } = config;
    const hash = hashTx(txdata, index, config);
    const sig = sign(prvkey, hash);
    return (sigflag === 0x00)
        ? Buff$1.raw(sig).hex
        : Buff$1.join([sig, sigflag]).hex;
}
async function verifyTx(txdata, index, config = {}, shouldThrow = false) {
    const tx = normalizeTx(txdata);
    const { witness } = tx.input[index];
    if (!Array.isArray(witness) || witness.length < 1) {
        return safeThrow('Invalid witness data: ' + String(witness), shouldThrow);
    }
    const annex = normalizeData(witness[witness.length - 1]);
    if (annex[0] === 0x50)
        witness.pop();
    if (witness.length < 1) {
        return safeThrow('Invalid witness data: ' + String(witness), shouldThrow);
    }
    const stream = new Stream(normalizeData(witness[0]));
    const signature = stream.read(64).raw;
    const prevout = tx.input[index].prevout;
    const tapkey = normalizeData(prevout?.scriptPubKey).slice(2);
    let target, cblock;
    if (stream.size === 1) {
        config.sigflag = stream.read(1).num;
        if (config.sigflag === 0x00) {
            return safeThrow('0x00 is not a valid appended sigflag!', shouldThrow);
        }
    }
    if (witness.length > 1) {
        cblock = normalizeData(witness.pop());
    }
    if (witness.length > 1 && cblock instanceof Uint8Array) {
        const script = encodeScript(witness.pop());
        const version = cblock[0] & 0xfe;
        target = await getTapLeaf(script, version);
        config.extension = target;
    }
    const hash = hashTx(tx, index, config);
    if (!verify(signature, hash, tapkey, true)) {
        return safeThrow('Invalid signature!', shouldThrow);
    }
    if (cblock !== undefined && target !== undefined &&
        !await checkTapPath(tapkey, cblock, target)) {
        return safeThrow('Invalid cblock!', shouldThrow);
    }
    return true;
}
function hashTx(txdata, index, config = {}) {
    if (typeof txdata === 'string' ||
        txdata instanceof Uint8Array) {
        txdata = decodeTx(txdata);
    }
    const { extension, sigflag = 0x00, extflag = 0x00, key_version = 0x00, separator_pos = 0xFFFFFFFF } = config;
    const { version, input = [], output = [], locktime } = txdata;
    if (index >= input.length) {
        throw new Error('Index out of bounds: ' + String(index));
    }
    if (!VALID_HASH_TYPES.includes(sigflag)) {
        throw new Error('Invalid hash type: ' + String(sigflag));
    }
    if (extflag < 0 || extflag > 127) {
        throw new Error('Extention flag out of range: ' + String(extflag));
    }
    const { txid, vout, sequence, witness = [] } = input[index];
    const isAnyPay = (sigflag & 0x80) === 0x80;
    const annex = getAnnexData(witness);
    const annexBit = (annex !== undefined) ? 1 : 0;
    const extendBit = (extension !== undefined) ? 1 : 0;
    const spendType = ((extflag + extendBit) * 2) + annexBit;
    const tag = Buff$1.str('TapSighash').digest;
    const digest = [
        tag,
        tag,
        Buff$1.num(0x00, 1),
        Buff$1.num(sigflag, 1),
        encodeVersion(version),
        encodeLocktime(locktime)
    ];
    if (!isAnyPay) {
        const prevouts = input.map(e => getPrevout(e));
        digest.push(hashOutpoints(input), hashAmounts(prevouts), hashScripts(prevouts), hashSequence(input));
    }
    if ((sigflag & 0x03) < 2 || (sigflag & 0x03) > 3) {
        digest.push(hashOutputs(output));
    }
    digest.push(Buff$1.num(spendType, 1));
    if (isAnyPay) {
        const { value, scriptPubKey } = getPrevout(input[index]);
        digest.push(encodeTxid(txid), encodePrevOut(vout), encodeValue(value), encodeScript(scriptPubKey), encodeSequence(sequence));
    }
    else {
        digest.push(Buff$1.num(index, 4).reverse());
    }
    if (annex !== undefined)
        digest.push(annex);
    if ((sigflag & 0x03) === 0x03) {
        digest.push(hashOutput(output[index]));
    }
    if (extension !== undefined) {
        digest.push(Buff$1.normalize(extension), Buff$1.num(key_version), Buff$1.num(separator_pos, 4));
    }
    return Buff$1.join(digest).digest;
}
function hashOutpoints(vin) {
    const stack = [];
    for (const { txid, vout } of vin) {
        stack.push(encodeTxid(txid));
        stack.push(encodePrevOut(vout));
    }
    return Buff$1.join(stack).digest;
}
function hashSequence(vin) {
    const stack = [];
    for (const { sequence } of vin) {
        stack.push(encodeSequence(sequence));
    }
    return Buff$1.join(stack).digest;
}
function hashAmounts(prevouts) {
    const stack = [];
    for (const { value } of prevouts) {
        stack.push(encodeValue(value));
    }
    return Buff$1.join(stack).digest;
}
function hashScripts(prevouts) {
    const stack = [];
    for (const { address, scriptPubKey } of prevouts) {
        if (typeof address === 'string') {
            const script = Address.convert(address);
            stack.push(encodeScript(script));
        }
        else {
            stack.push(encodeScript(scriptPubKey));
        }
    }
    return Buff$1.join(stack).digest;
}
function hashOutputs(vout) {
    const stack = [];
    for (const { value, scriptPubKey } of vout) {
        stack.push(encodeValue(value));
        stack.push(encodeScript(scriptPubKey));
    }
    return Buff$1.join(stack).digest;
}
function hashOutput(vout) {
    return Buff$1.join([
        encodeValue(vout.value),
        encodeScript(vout.scriptPubKey)
    ]).digest;
}
function getAnnexData(witness) {
    if (witness === undefined)
        return;
    if (witness.length < 2)
        return;
    let annex = witness.at(-1);
    if (typeof annex === 'string') {
        annex = Buff$1.hex(annex);
    }
    if (annex instanceof Uint8Array &&
        annex[0] === 0x50) {
        return Buff$1.raw(annex).prefixSize('be').digest;
    }
    return undefined;
}
function getPrevout(vin) {
    if (vin.prevout === undefined) {
        throw new Error('Prevout data missing for input: ' + String(vin.txid));
    }
    return vin.prevout;
}

const Script = {
    encode: encodeScript,
    decode: decodeScript
};
const Sig = {
    segwit: {
        hash: segwitHash
    },
    taproot: {
        hash: hashTx,
        sign: signTx,
        verify: verifyTx
    }
};
const Tree = {
    getTag: getTapTag,
    getLeaf: getTapLeaf,
    getBranch: getTapBranch,
    getRoot: getTapRoot,
    getPath: getTapPath,
    checkPath: checkTapPath,
    encodeAddress: encodeTapAddress,
    decodeAddress: decodeTapAddress
};
const Tweak = {
    getPubkey: getTapPubkey,
    getSeckey: getTapSeckey,
    getTweak: getTapTweak,
    tweakSeckey: tweakPrvkey,
    tweakPubkey: tweakPubkey
};
const Tx = {
    encode: encodeTx,
    decode: decodeTx
};

export { Address, Script, Sig, Tree, Tweak, Tx };
//# sourceMappingURL=module.js.map
