import { encodeTx } from './lib/tx/encode.js';
import { decodeTx } from './lib/tx/decode.js';
import { encodeScript } from './lib/script/encode.js';
import { decodeScript } from './lib/script/decode.js';
import { segwitHash } from './lib/sig/segwit.js';
import * as TAPSIG from './lib/sig/taproot.js';
import * as TAPSCR from './lib/tree/script.js';
import * as TAPTWK from './lib/tree/tweak.js';
import * as TAPCHK from './lib/tree/proof.js';
export { Address } from './lib/addr/index.js';
export * from './schema/types.js';
export declare const Script: {
    encode: typeof encodeScript;
    decode: typeof decodeScript;
};
export declare const Sig: {
    segwit: {
        hash: typeof segwitHash;
    };
    taproot: {
        hash: typeof TAPSIG.hashTx;
        sign: typeof TAPSIG.signTx;
        verify: typeof TAPSIG.verifyTx;
    };
};
export declare const Tree: {
    getTag: typeof TAPSCR.getTapTag;
    getLeaf: typeof TAPSCR.getTapLeaf;
    getBranch: typeof TAPSCR.getTapBranch;
    getRoot: typeof TAPSCR.getTapRoot;
    getPath: typeof TAPCHK.getTapPath;
    checkPath: typeof TAPCHK.checkTapPath;
    encodeAddress: typeof TAPSCR.encodeTapAddress;
    decodeAddress: typeof TAPSCR.decodeTapAddress;
};
export declare const Tweak: {
    getPubkey: typeof TAPTWK.getTapPubkey;
    getSeckey: typeof TAPTWK.getTapSeckey;
    getTweak: typeof TAPTWK.getTapTweak;
    tweakSeckey: typeof TAPTWK.tweakPrvkey;
    tweakPubkey: typeof TAPTWK.tweakPubkey;
};
export declare const Tx: {
    encode: typeof encodeTx;
    decode: typeof decodeTx;
};
//# sourceMappingURL=index.d.ts.map